package javawrapper;

import junit.framework.*;
import java.io.FileWriter;
import java.io.File;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.InputSource;

/**
 * Tests the Grid class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestGrid
    extends TestCase {

  /**
   * Tests the retrieval and addition of GridVal objects.
   */
  public void testGetGridValue() {
    try {
      Grid oGrid = new Grid("test grid",
                            new DataMember[] {new DataMember("test", "test",
          DataMember.BOOLEAN)}
                            , //data member list
                            new DataMember[] {new DataMember("testpackage",
          "testpackage", DataMember.FLOAT)}
                            , //package data member list
                            0, 0);

      //Normal processing
      Plot oPlot = new Plot(null);
      oPlot.SetPlotXLength(200);
      oPlot.SetPlotYLength(200);

      //Case:  no grid values yet
      assertEquals(0, oGrid.mp_oGridVals.size());
      GridValue oVal = oGrid.GetGridValue(0, 10, oPlot);
      assertEquals(1, oGrid.mp_oGridVals.size());

      //Case:  no packages in grid
      assertEquals(0, oVal.mp_oPackages.size());
      //Adding a package
      PackageGridValue oPackage = oGrid.GetPackageGridValue(oVal, 0);
      assertEquals(1, oVal.mp_oPackages.size());
      //Set a value
      oPackage.SetValue(0, new Float(10));

      //Case: adding a new one
      oVal = oGrid.GetGridValue(1, 10, oPlot);
      assertEquals(2, oGrid.mp_oGridVals.size());
      //Make sure packages get assigned correctly
      oGrid.GetPackageGridValue(oVal, 0);
      assertEquals(1, oVal.mp_oPackages.size());

      //Finds an existing
      oVal = oGrid.GetGridValue(0, 10, oPlot);
      assertEquals(2, oGrid.mp_oGridVals.size());
      assertEquals(0, oVal.GetCell().GetX());
      assertEquals(10, oVal.GetCell().GetY());
      assertEquals( (float) 8, oVal.GetCell().m_fXCellLength, 0.00000001);
      assertEquals( (float) 8, oVal.GetCell().m_fYCellLength, 0.00000001);
      assertEquals(1, oVal.mp_oPackages.size());
      oPackage = oGrid.GetPackageGridValue(oVal, 0);

      //New one when there have been grid values set - this should never happen
      //this way in real life but saves the trouble of going back and starting
      //over
      oGrid.SetXCellLength( (float) 12.3);
      oGrid.SetYCellLength( (float) 15.2);
      oVal = oGrid.GetGridValue(5, 6, oPlot);
      assertEquals(3, oGrid.mp_oGridVals.size());
      assertEquals(5, oVal.GetCell().GetX());
      assertEquals(6, oVal.GetCell().GetY());
      assertEquals( (float) 12.3, oVal.GetCell().m_fXCellLength, 0.00000001);
      assertEquals( (float) 15.2, oVal.GetCell().m_fYCellLength, 0.00000001);

      //Case:  all data member types present
      float fXCellLength = 8, fYCellLength = 8;
      oGrid = null;
      DataMember[] p_oDataMembers = new DataMember[] {

          new DataMember("bool 1", "bool 1", DataMember.BOOLEAN),
          new DataMember("float 1", "float 1", DataMember.FLOAT),
          new DataMember("bool 2", "bool 2", DataMember.BOOLEAN),
          new DataMember("int 1", "int 1", DataMember.INTEGER),
          new DataMember("int 2", "int 2", DataMember.INTEGER),
          new DataMember("char 1", "char 1", DataMember.CHAR),
          new DataMember("char 2", "char 2", DataMember.CHAR),
          new DataMember("bool 3", "bool 3", DataMember.BOOLEAN),
          new DataMember("bool 4", "bool 4", DataMember.BOOLEAN),
          new DataMember("char 3", "char 3", DataMember.CHAR)
      };
      DataMember[] p_oPackageDataMembers = new DataMember[] {

          new DataMember("package bool 1", "package bool 1", DataMember.BOOLEAN),
          new DataMember("package float 1", "package float 1", DataMember.FLOAT),
          new DataMember("package float 2", "package float 2", DataMember.FLOAT),
          new DataMember("package bool 2", "package bool 2", DataMember.BOOLEAN),
          new DataMember("package int 1", "package int 1", DataMember.INTEGER),
          new DataMember("package int 2", "package int 2", DataMember.INTEGER),
          new DataMember("package int 3", "package int 3", DataMember.INTEGER),
          new DataMember("package char 1", "package char 1", DataMember.CHAR),
          new DataMember("package char 2", "package char 2", DataMember.CHAR),
          new DataMember("package bool 3", "package bool 3", DataMember.BOOLEAN),
          new DataMember("package bool 4", "package bool 4", DataMember.BOOLEAN),
          new DataMember("package bool 5", "package bool 5", DataMember.BOOLEAN),
          new DataMember("package char 3", "package char 3", DataMember.CHAR),
          new DataMember("package char 4", "package char 4", DataMember.CHAR)
      };
      oGrid = new Grid("test", p_oDataMembers, p_oPackageDataMembers,
                       fXCellLength, fYCellLength);
      oVal = oGrid.GetGridValue(5, 6, oPlot);
      oPackage = oGrid.GetPackageGridValue(oVal, 0);
      assertEquals(1, oVal.mp_fFloats.length);
      assertEquals(2, oVal.mp_iInts.length);
      assertEquals(3, oVal.mp_sChars.length);
      assertEquals(4, oVal.mp_bBools.length);
      assertEquals(2, oPackage.mp_fFloats.length);
      assertEquals(3, oPackage.mp_iInts.length);
      assertEquals(4, oPackage.mp_sChars.length);
      assertEquals(5, oPackage.mp_bBools.length);

      //Case: no floats - cell
      //Case: no bools - package
      oGrid = null;
      p_oDataMembers = null;
      p_oPackageDataMembers = null;
      p_oDataMembers = new DataMember[] {

          new DataMember("bool 1", "bool 1", DataMember.BOOLEAN),
          new DataMember("bool 2", "bool 2", DataMember.BOOLEAN),
          new DataMember("int 1", "int 1", DataMember.INTEGER),
          new DataMember("int 2", "int 2", DataMember.INTEGER),
          new DataMember("char 1", "char 1", DataMember.CHAR),
          new DataMember("char 2", "char 2", DataMember.CHAR),
          new DataMember("bool 3", "bool 3", DataMember.BOOLEAN),
          new DataMember("bool 4", "bool 4", DataMember.BOOLEAN),
          new DataMember("char 3", "char 3", DataMember.CHAR)
      };
      p_oPackageDataMembers = new DataMember[] {

          new DataMember("package float 1", "package float 1", DataMember.FLOAT),
          new DataMember("package float 2", "package float 2", DataMember.FLOAT),
          new DataMember("package int 1", "package int 1", DataMember.INTEGER),
          new DataMember("package int 2", "package int 2", DataMember.INTEGER),
          new DataMember("package int 3", "package int 3", DataMember.INTEGER),
          new DataMember("package char 1", "package char 1", DataMember.CHAR),
          new DataMember("package char 2", "package char 2", DataMember.CHAR),
          new DataMember("package char 3", "package char 3", DataMember.CHAR),
          new DataMember("package char 4", "package char 4", DataMember.CHAR)
      };
      oGrid = new Grid("test", p_oDataMembers, p_oPackageDataMembers,
                       fXCellLength, fYCellLength);
      oVal = oGrid.GetGridValue(5, 6, oPlot);
      oPackage = oGrid.GetPackageGridValue(oVal, 0);
      assertEquals(0, oVal.mp_fFloats.length);
      assertEquals(2, oVal.mp_iInts.length);
      assertEquals(3, oVal.mp_sChars.length);
      assertEquals(4, oVal.mp_bBools.length);
      assertEquals(2, oPackage.mp_fFloats.length);
      assertEquals(3, oPackage.mp_iInts.length);
      assertEquals(4, oPackage.mp_sChars.length);
      assertEquals(0, oPackage.mp_bBools.length);

      //Case: no ints - cell
      //Case: no chars - package
      oGrid = null;
      p_oDataMembers = null;
      p_oPackageDataMembers = null;
      p_oDataMembers = new DataMember[] {

          new DataMember("bool 1", "bool 1", DataMember.BOOLEAN),
          new DataMember("float 1", "float 1", DataMember.FLOAT),
          new DataMember("bool 2", "bool 2", DataMember.BOOLEAN),
          new DataMember("char 1", "char 1", DataMember.CHAR),
          new DataMember("char 2", "char 2", DataMember.CHAR),
          new DataMember("bool 3", "bool 3", DataMember.BOOLEAN),
          new DataMember("bool 4", "bool 4", DataMember.BOOLEAN),
          new DataMember("char 3", "char 3", DataMember.CHAR)
      };
      p_oPackageDataMembers = new DataMember[] {

          new DataMember("package bool 1", "package bool 1", DataMember.BOOLEAN),
          new DataMember("package float 1", "package float 1", DataMember.FLOAT),
          new DataMember("package float 2", "package float 2", DataMember.FLOAT),
          new DataMember("package bool 2", "package bool 2", DataMember.BOOLEAN),
          new DataMember("package int 1", "package int 1", DataMember.INTEGER),
          new DataMember("package int 2", "package int 2", DataMember.INTEGER),
          new DataMember("package int 3", "package int 3", DataMember.INTEGER),
          new DataMember("package bool 3", "package bool 3", DataMember.BOOLEAN),
          new DataMember("package bool 4", "package bool 4", DataMember.BOOLEAN),
          new DataMember("package bool 5", "package bool 5", DataMember.BOOLEAN),
      };
      oGrid = new Grid("test", p_oDataMembers, p_oPackageDataMembers,
                       fXCellLength, fYCellLength);
      oVal = oGrid.GetGridValue(5, 6, oPlot);
      oPackage = oGrid.GetPackageGridValue(oVal, 0);
      assertEquals(1, oVal.mp_fFloats.length);
      assertEquals(0, oVal.mp_iInts.length);
      assertEquals(3, oVal.mp_sChars.length);
      assertEquals(4, oVal.mp_bBools.length);
      assertEquals(2, oPackage.mp_fFloats.length);
      assertEquals(3, oPackage.mp_iInts.length);
      assertEquals(0, oPackage.mp_sChars.length);
      assertEquals(5, oPackage.mp_bBools.length);

      //Case: no chars - cell
      //Case: no floats - package
      oGrid = null;
      p_oDataMembers = null;
      p_oPackageDataMembers = null;
      p_oDataMembers = new DataMember[] {

          new DataMember("bool 1", "bool 1", DataMember.BOOLEAN),
          new DataMember("float 1", "float 1", DataMember.FLOAT),
          new DataMember("bool 2", "bool 2", DataMember.BOOLEAN),
          new DataMember("int 1", "int 1", DataMember.INTEGER),
          new DataMember("int 2", "int 2", DataMember.INTEGER),
          new DataMember("bool 3", "bool 3", DataMember.BOOLEAN),
          new DataMember("bool 4", "bool 4", DataMember.BOOLEAN),
      };
      p_oPackageDataMembers = new DataMember[] {

          new DataMember("package bool 1", "package bool 1", DataMember.BOOLEAN),
          new DataMember("package bool 2", "package bool 2", DataMember.BOOLEAN),
          new DataMember("package int 1", "package int 1", DataMember.INTEGER),
          new DataMember("package int 2", "package int 2", DataMember.INTEGER),
          new DataMember("package int 3", "package int 3", DataMember.INTEGER),
          new DataMember("package char 1", "package char 1", DataMember.CHAR),
          new DataMember("package char 2", "package char 2", DataMember.CHAR),
          new DataMember("package bool 3", "package bool 3", DataMember.BOOLEAN),
          new DataMember("package bool 4", "package bool 4", DataMember.BOOLEAN),
          new DataMember("package bool 5", "package bool 5", DataMember.BOOLEAN),
          new DataMember("package char 3", "package char 3", DataMember.CHAR),
          new DataMember("package char 4", "package char 4", DataMember.CHAR)
      };
      oGrid = new Grid("test", p_oDataMembers, p_oPackageDataMembers,
                       fXCellLength, fYCellLength);
      oVal = oGrid.GetGridValue(5, 6, oPlot);
      oPackage = oGrid.GetPackageGridValue(oVal, 0);
      assertEquals(1, oVal.mp_fFloats.length);
      assertEquals(2, oVal.mp_iInts.length);
      assertEquals(0, oVal.mp_sChars.length);
      assertEquals(4, oVal.mp_bBools.length);
      assertEquals(0, oPackage.mp_fFloats.length);
      assertEquals(3, oPackage.mp_iInts.length);
      assertEquals(4, oPackage.mp_sChars.length);
      assertEquals(5, oPackage.mp_bBools.length);

      //Case: no bools - cell
      //Case: no ints - package
      oGrid = null;
      p_oDataMembers = null;
      p_oPackageDataMembers = null;
      p_oDataMembers = new DataMember[] {

          new DataMember("float 1", "float 1", DataMember.FLOAT),
          new DataMember("int 1", "int 1", DataMember.INTEGER),
          new DataMember("int 2", "int 2", DataMember.INTEGER),
          new DataMember("char 1", "char 1", DataMember.CHAR),
          new DataMember("char 2", "char 2", DataMember.CHAR),
          new DataMember("char 3", "char 3", DataMember.CHAR)
      };
      p_oPackageDataMembers = new DataMember[] {

          new DataMember("package bool 1", "package bool 1", DataMember.BOOLEAN),
          new DataMember("package float 1", "package float 1", DataMember.FLOAT),
          new DataMember("package float 2", "package float 2", DataMember.FLOAT),
          new DataMember("package bool 2", "package bool 2", DataMember.BOOLEAN),
          new DataMember("package char 1", "package char 1", DataMember.CHAR),
          new DataMember("package char 2", "package char 2", DataMember.CHAR),
          new DataMember("package bool 3", "package bool 3", DataMember.BOOLEAN),
          new DataMember("package bool 4", "package bool 4", DataMember.BOOLEAN),
          new DataMember("package bool 5", "package bool 5", DataMember.BOOLEAN),
          new DataMember("package char 3", "package char 3", DataMember.CHAR),
          new DataMember("package char 4", "package char 4", DataMember.CHAR)
      };
      oGrid = new Grid("test", p_oDataMembers, p_oPackageDataMembers,
                       fXCellLength, fYCellLength);
      oVal = oGrid.GetGridValue(5, 6, oPlot);
      oPackage = oGrid.GetPackageGridValue(oVal, 0);
      assertEquals(1, oVal.mp_fFloats.length);
      assertEquals(2, oVal.mp_iInts.length);
      assertEquals(3, oVal.mp_sChars.length);
      assertEquals(0, oVal.mp_bBools.length);
      assertEquals(2, oPackage.mp_fFloats.length);
      assertEquals(0, oPackage.mp_iInts.length);
      assertEquals(4, oPackage.mp_sChars.length);
      assertEquals(5, oPackage.mp_bBools.length);

      System.out.println("GetGridValue testing succeeded.");
    }
    catch (ModelException oErr) {
      fail("GetGridValue failed with message " + oErr.getMessage());
    }
  }

  /**
   * Tests the Grid constructor.
   */
  public void testGrid() {
    try {
      //Case:  Some of each type of data member.
      DataMember[] p_oDataMembers = new DataMember[] {

          new DataMember("bool 1", "bool 1", DataMember.BOOLEAN),
          new DataMember("float 1", "float 1", DataMember.FLOAT),
          new DataMember("bool 2", "bool 2", DataMember.BOOLEAN),
          new DataMember("int 1", "int 1", DataMember.INTEGER),
          new DataMember("int 2", "int 2", DataMember.INTEGER),
          new DataMember("char 1", "char 1", DataMember.CHAR),
          new DataMember("char 2", "char 2", DataMember.CHAR),
          new DataMember("bool 3", "bool 3", DataMember.BOOLEAN),
          new DataMember("bool 4", "bool 4", DataMember.BOOLEAN),
          new DataMember("char 3", "char 3", DataMember.CHAR)
      };
      DataMember[] p_oPackageDataMembers = new DataMember[] {

          new DataMember("package bool 1", "package bool 1", DataMember.BOOLEAN),
          new DataMember("package float 1", "package float 1", DataMember.FLOAT),
          new DataMember("package float 2", "package float 2", DataMember.FLOAT),
          new DataMember("package bool 2", "package bool 2", DataMember.BOOLEAN),
          new DataMember("package int 1", "package int 1", DataMember.INTEGER),
          new DataMember("package int 2", "package int 2", DataMember.INTEGER),
          new DataMember("package int 3", "package int 3", DataMember.INTEGER),
          new DataMember("package char 1", "package char 1", DataMember.CHAR),
          new DataMember("package char 2", "package char 2", DataMember.CHAR),
          new DataMember("package bool 3", "package bool 3", DataMember.BOOLEAN),
          new DataMember("package bool 4", "package bool 4", DataMember.BOOLEAN),
          new DataMember("package bool 5", "package bool 5", DataMember.BOOLEAN),
          new DataMember("package char 3", "package char 3", DataMember.CHAR),
          new DataMember("package char 4", "package char 4", DataMember.CHAR)
      };

      Grid oGrid = new Grid("test grid", p_oDataMembers, p_oPackageDataMembers,
                            8, 8);
      assertEquals(1, oGrid.mp_sFloatDataMembers.length);
      assertEquals(2, oGrid.mp_sIntDataMembers.length);
      assertEquals(3, oGrid.mp_sCharDataMembers.length);
      assertEquals(4, oGrid.mp_sBoolDataMembers.length);

      assertEquals(2, oGrid.mp_sPackageFloatDataMembers.length);
      assertEquals(3, oGrid.mp_sPackageIntDataMembers.length);
      assertEquals(4, oGrid.mp_sPackageCharDataMembers.length);
      assertEquals(5, oGrid.mp_sPackageBoolDataMembers.length);

      assertEquals(1, oGrid.mp_iGridFloatTransforms.size());
      assertEquals(2, oGrid.mp_iGridIntTransforms.size());
      assertEquals(3, oGrid.mp_iGridCharTransforms.size());
      assertEquals(4, oGrid.mp_iGridBoolTransforms.size());

      assertEquals(2, oGrid.mp_iGridPackageFloatTransforms.size());
      assertEquals(3, oGrid.mp_iGridPackageIntTransforms.size());
      assertEquals(4, oGrid.mp_iGridPackageCharTransforms.size());
      assertEquals(5, oGrid.mp_iGridPackageBoolTransforms.size());

      assertEquals("float 1", oGrid.mp_sFloatDataMembers[0]);
      Integer iTest = (Integer) oGrid.mp_iGridFloatTransforms.elementAt(0);
      assertEquals(iTest.intValue(), 0);

      assertEquals("int 1", oGrid.mp_sIntDataMembers[0]);
      iTest = (Integer) oGrid.mp_iGridIntTransforms.elementAt(0);
      assertEquals(iTest.intValue(), 0);
      assertEquals("int 2", oGrid.mp_sIntDataMembers[1]);
      iTest = (Integer) oGrid.mp_iGridIntTransforms.elementAt(1);
      assertEquals(iTest.intValue(), 1);

      assertEquals("char 1", oGrid.mp_sCharDataMembers[0]);
      iTest = (Integer) oGrid.mp_iGridCharTransforms.elementAt(0);
      assertEquals(iTest.intValue(), 0);
      assertEquals("char 2", oGrid.mp_sCharDataMembers[1]);
      iTest = (Integer) oGrid.mp_iGridCharTransforms.elementAt(1);
      assertEquals(iTest.intValue(), 1);
      assertEquals("char 3", oGrid.mp_sCharDataMembers[2]);
      iTest = (Integer) oGrid.mp_iGridCharTransforms.elementAt(2);
      assertEquals(iTest.intValue(), 2);

      assertEquals("bool 1", oGrid.mp_sBoolDataMembers[0]);
      iTest = (Integer) oGrid.mp_iGridBoolTransforms.elementAt(0);
      assertEquals(iTest.intValue(), 0);
      assertEquals("bool 2", oGrid.mp_sBoolDataMembers[1]);
      iTest = (Integer) oGrid.mp_iGridBoolTransforms.elementAt(1);
      assertEquals(iTest.intValue(), 1);
      assertEquals("bool 3", oGrid.mp_sBoolDataMembers[2]);
      iTest = (Integer) oGrid.mp_iGridBoolTransforms.elementAt(2);
      assertEquals(iTest.intValue(), 2);
      assertEquals("bool 4", oGrid.mp_sBoolDataMembers[3]);
      iTest = (Integer) oGrid.mp_iGridBoolTransforms.elementAt(3);
      assertEquals(iTest.intValue(), 3);

      assertEquals("package float 1", oGrid.mp_sPackageFloatDataMembers[0]);
      iTest = (Integer) oGrid.mp_iGridPackageFloatTransforms.elementAt(0);
      assertEquals(iTest.intValue(), 0);
      assertEquals("package float 2", oGrid.mp_sPackageFloatDataMembers[1]);
      iTest = (Integer) oGrid.mp_iGridPackageFloatTransforms.elementAt(1);
      assertEquals(iTest.intValue(), 1);

      assertEquals("package int 1", oGrid.mp_sPackageIntDataMembers[0]);
      iTest = (Integer) oGrid.mp_iGridPackageIntTransforms.elementAt(0);
      assertEquals(iTest.intValue(), 0);
      assertEquals("package int 2", oGrid.mp_sPackageIntDataMembers[1]);
      iTest = (Integer) oGrid.mp_iGridPackageIntTransforms.elementAt(1);
      assertEquals(iTest.intValue(), 1);
      assertEquals("package int 3", oGrid.mp_sPackageIntDataMembers[2]);
      iTest = (Integer) oGrid.mp_iGridPackageIntTransforms.elementAt(2);
      assertEquals(iTest.intValue(), 2);

      assertEquals("package char 1", oGrid.mp_sPackageCharDataMembers[0]);
      iTest = (Integer) oGrid.mp_iGridPackageCharTransforms.elementAt(0);
      assertEquals(iTest.intValue(), 0);
      assertEquals("package char 2", oGrid.mp_sPackageCharDataMembers[1]);
      iTest = (Integer) oGrid.mp_iGridPackageCharTransforms.elementAt(1);
      assertEquals(iTest.intValue(), 1);
      assertEquals("package char 3", oGrid.mp_sPackageCharDataMembers[2]);
      iTest = (Integer) oGrid.mp_iGridPackageCharTransforms.elementAt(2);
      assertEquals(iTest.intValue(), 2);
      assertEquals("package char 4", oGrid.mp_sPackageCharDataMembers[3]);
      iTest = (Integer) oGrid.mp_iGridPackageCharTransforms.elementAt(3);
      assertEquals(iTest.intValue(), 3);

      assertEquals("package bool 1", oGrid.mp_sPackageBoolDataMembers[0]);
      iTest = (Integer) oGrid.mp_iGridPackageBoolTransforms.elementAt(0);
      assertEquals(iTest.intValue(), 0);
      assertEquals("package bool 2", oGrid.mp_sPackageBoolDataMembers[1]);
      iTest = (Integer) oGrid.mp_iGridPackageBoolTransforms.elementAt(1);
      assertEquals(iTest.intValue(), 1);
      assertEquals("package bool 3", oGrid.mp_sPackageBoolDataMembers[2]);
      iTest = (Integer) oGrid.mp_iGridPackageBoolTransforms.elementAt(2);
      assertEquals(iTest.intValue(), 2);
      assertEquals("package bool 4", oGrid.mp_sPackageBoolDataMembers[3]);
      iTest = (Integer) oGrid.mp_iGridPackageBoolTransforms.elementAt(3);
      assertEquals(iTest.intValue(), 3);
      assertEquals("package bool 5", oGrid.mp_sPackageBoolDataMembers[4]);
      iTest = (Integer) oGrid.mp_iGridPackageBoolTransforms.elementAt(4);
      assertEquals(iTest.intValue(), 4);

      //Case:  not all data members
      oGrid = null;
      p_oDataMembers = null;
      p_oPackageDataMembers = null;
      p_oDataMembers = new DataMember[] {

          new DataMember("bool 1", "bool 1", DataMember.BOOLEAN),
          new DataMember("float 1", "float 1", DataMember.FLOAT),
          new DataMember("bool 2", "bool 2", DataMember.BOOLEAN),
          new DataMember("int 1", "int 1", DataMember.INTEGER),
          new DataMember("int 2", "int 2", DataMember.INTEGER),
          new DataMember("bool 3", "bool 3", DataMember.BOOLEAN),
          new DataMember("bool 4", "bool 4", DataMember.BOOLEAN),
      };

      p_oPackageDataMembers = new DataMember[] {

          new DataMember("package float 1", "package float 1", DataMember.FLOAT),
          new DataMember("package float 2", "package float 2", DataMember.FLOAT),
          new DataMember("package int 1", "package int 1", DataMember.INTEGER),
          new DataMember("package int 2", "package int 2", DataMember.INTEGER),
          new DataMember("package int 3", "package int 3", DataMember.INTEGER),
          new DataMember("package char 1", "package char 1", DataMember.CHAR),
          new DataMember("package char 2", "package char 2", DataMember.CHAR),
          new DataMember("package char 3", "package char 3", DataMember.CHAR),
          new DataMember("package char 4", "package char 4", DataMember.CHAR)
      };

      oGrid = new Grid("test grid", p_oDataMembers, p_oPackageDataMembers, 8, 8);
      assertEquals(1, oGrid.mp_sFloatDataMembers.length);
      assertEquals(2, oGrid.mp_sIntDataMembers.length);
      assertTrue(null == oGrid.mp_sCharDataMembers);
      assertEquals(4, oGrid.mp_sBoolDataMembers.length);

      assertEquals(2, oGrid.mp_sPackageFloatDataMembers.length);
      assertEquals(3, oGrid.mp_sPackageIntDataMembers.length);
      assertEquals(4, oGrid.mp_sPackageCharDataMembers.length);
      assertTrue(null == oGrid.mp_sPackageBoolDataMembers);

      assertEquals("float 1", oGrid.mp_sFloatDataMembers[0]);

      assertEquals("int 1", oGrid.mp_sIntDataMembers[0]);
      assertEquals("int 2", oGrid.mp_sIntDataMembers[1]);

      assertEquals("bool 1", oGrid.mp_sBoolDataMembers[0]);
      assertEquals("bool 2", oGrid.mp_sBoolDataMembers[1]);
      assertEquals("bool 3", oGrid.mp_sBoolDataMembers[2]);
      assertEquals("bool 4", oGrid.mp_sBoolDataMembers[3]);

      assertEquals("package float 1", oGrid.mp_sPackageFloatDataMembers[0]);
      assertEquals("package float 2", oGrid.mp_sPackageFloatDataMembers[1]);

      assertEquals("package int 1", oGrid.mp_sPackageIntDataMembers[0]);
      assertEquals("package int 2", oGrid.mp_sPackageIntDataMembers[1]);
      assertEquals("package int 3", oGrid.mp_sPackageIntDataMembers[2]);

      assertEquals("package char 1", oGrid.mp_sPackageCharDataMembers[0]);
      assertEquals("package char 2", oGrid.mp_sPackageCharDataMembers[1]);
      assertEquals("package char 3", oGrid.mp_sPackageCharDataMembers[2]);
      assertEquals("package char 4", oGrid.mp_sPackageCharDataMembers[3]);

      //Case: no package data members
      oGrid = null;
      p_oDataMembers = null;
      p_oPackageDataMembers = null;
      p_oDataMembers = new DataMember[] {

          new DataMember("bool 1", "bool 1", DataMember.BOOLEAN),
          new DataMember("float 1", "float 1", DataMember.FLOAT),
          new DataMember("bool 2", "bool 2", DataMember.BOOLEAN),
          new DataMember("int 1", "int 1", DataMember.INTEGER),
          new DataMember("int 2", "int 2", DataMember.INTEGER),
          new DataMember("bool 3", "bool 3", DataMember.BOOLEAN),
          new DataMember("bool 4", "bool 4", DataMember.BOOLEAN),
      };

      oGrid = new Grid("test grid", p_oDataMembers, p_oPackageDataMembers, 8, 8);
      assertEquals(1, oGrid.mp_sFloatDataMembers.length);
      assertEquals(2, oGrid.mp_sIntDataMembers.length);
      assertTrue(null == oGrid.mp_sCharDataMembers);
      assertEquals(4, oGrid.mp_sBoolDataMembers.length);

      assertTrue(null == oGrid.mp_sPackageFloatDataMembers);
      assertTrue(null == oGrid.mp_sPackageIntDataMembers);
      assertTrue(null == oGrid.mp_sPackageCharDataMembers);
      assertTrue(null == oGrid.mp_sPackageBoolDataMembers);

      assertEquals("float 1", oGrid.mp_sFloatDataMembers[0]);

      assertEquals("int 1", oGrid.mp_sIntDataMembers[0]);
      assertEquals("int 2", oGrid.mp_sIntDataMembers[1]);

      assertEquals("bool 1", oGrid.mp_sBoolDataMembers[0]);
      assertEquals("bool 2", oGrid.mp_sBoolDataMembers[1]);
      assertEquals("bool 3", oGrid.mp_sBoolDataMembers[2]);
      assertEquals("bool 4", oGrid.mp_sBoolDataMembers[3]);

      System.out.println("Grid constructor testing succeeded.");

    }
    catch (ModelException oErr) {
      fail("Constructor testing failed with message " + oErr.getMessage());
    }
  }

  /**
   * Tests setting the various types of grid values.
   */
  public void testSettingGridValues() {
    try {
      Plot oPlot = new Plot(null);
      oPlot.SetPlotXLength(200);
      oPlot.SetPlotYLength(200);

      //Case:  Some of each type of data member.
      DataMember[] p_oDataMembers = new DataMember[] {

          new DataMember("bool 1", "bool 1", DataMember.BOOLEAN),
          new DataMember("float 1", "float 1", DataMember.FLOAT),
          new DataMember("bool 2", "bool 2", DataMember.BOOLEAN),
          new DataMember("int 1", "int 1", DataMember.INTEGER),
          new DataMember("int 2", "int 2", DataMember.INTEGER),
          new DataMember("char 1", "char 1", DataMember.CHAR),
          new DataMember("char 2", "char 2", DataMember.CHAR),
          new DataMember("bool 3", "bool 3", DataMember.BOOLEAN),
          new DataMember("bool 4", "bool 4", DataMember.BOOLEAN),
          new DataMember("char 3", "char 3", DataMember.CHAR)
      };

      DataMember[] p_oPackageDataMembers = new DataMember[] {

          new DataMember("package bool 1", "package bool 1", DataMember.BOOLEAN),
          new DataMember("package float 1", "package float 1", DataMember.FLOAT),
          new DataMember("package float 2", "package float 2", DataMember.FLOAT),
          new DataMember("package bool 2", "package bool 2", DataMember.BOOLEAN),
          new DataMember("package int 1", "package int 1", DataMember.INTEGER),
          new DataMember("package int 2", "package int 2", DataMember.INTEGER),
          new DataMember("package int 3", "package int 3", DataMember.INTEGER),
          new DataMember("package char 1", "package char 1", DataMember.CHAR),
          new DataMember("package char 2", "package char 2", DataMember.CHAR),
          new DataMember("package bool 3", "package bool 3", DataMember.BOOLEAN),
          new DataMember("package bool 4", "package bool 4", DataMember.BOOLEAN),
          new DataMember("package bool 5", "package bool 5", DataMember.BOOLEAN),
          new DataMember("package char 3", "package char 3", DataMember.CHAR),
          new DataMember("package char 4", "package char 4", DataMember.CHAR)
      };

      Grid oGrid = new Grid("test grid", p_oDataMembers, p_oPackageDataMembers,
                            8, 8);

      //Straight up value setting
      int iX = 2, iY = 3, iFloatCode = 0, iIntCode = 1, iCharCode = 2,
          iBoolCode = 3;
      oGrid.SetGridValue(iX, iY, iFloatCode, new Float(0.1), oPlot);
      oGrid.SetGridValue(iX, iY, iIntCode, new Integer(1), oPlot);
      oGrid.SetGridValue(iX, iY, iCharCode, "test val 1", oPlot);
      oGrid.SetGridValue(iX, iY, iBoolCode, new Boolean(true), oPlot);

      iFloatCode = 1;
      iIntCode = 2;
      iCharCode = 3;
      iBoolCode = 4;
      oGrid.SetGridPackageValue(iX, iY, 0, iFloatCode, new Float(0.2), oPlot);
      oGrid.SetGridPackageValue(iX, iY, 0, iIntCode, new Integer(2), oPlot);
      oGrid.SetGridPackageValue(iX, iY, 0, iCharCode, "test val 2", oPlot);
      oGrid.SetGridPackageValue(iX, iY, 0, iBoolCode, new Boolean(true), oPlot);

      iFloatCode = 0;
      iIntCode = 1;
      iCharCode = 2;
      iBoolCode = 3;
      GridValue oVal = oGrid.GetGridValue(iX, iY, oPlot);
      assertEquals(0.1, oVal.mp_fFloats[iFloatCode].floatValue(), 0.01);
      assertEquals(1, oVal.mp_iInts[iIntCode].intValue());
      assertEquals("test val 1", oVal.mp_sChars[iCharCode]);
      assertEquals(true, oVal.mp_bBools[iBoolCode].booleanValue());

      iFloatCode = 1;
      iIntCode = 2;
      iCharCode = 3;
      iBoolCode = 4;
      PackageGridValue oPackage = oGrid.GetPackageGridValue(oVal, 0);
      assertEquals(0.2, oPackage.mp_fFloats[iFloatCode].floatValue(), 0.01);
      assertEquals(2, oPackage.mp_iInts[iIntCode].intValue());
      assertEquals("test val 2", oPackage.mp_sChars[iCharCode]);
      assertEquals(true, oPackage.mp_bBools[iBoolCode].booleanValue());

      //Mess with the transforms
      oGrid.SetGridBoolCode("bool 1", 5);
      oGrid.SetGridFloatCode("float 1", 2);
      oGrid.SetGridCharCode("char 1", 4);
      oGrid.SetGridIntCode("int 1", 3);

      oGrid.SetGridPackageBoolCode("package bool 1", 6);
      oGrid.SetGridPackageFloatCode("package float 1", 3);
      oGrid.SetGridPackageCharCode("package char 1", 5);
      oGrid.SetGridPackageIntCode("package int 1", 4);

      iX = 4;
      iY = 5;
      oGrid.SetGridValue(iX, iY, 2, new Float(0.1), oPlot);
      oGrid.SetGridValue(iX, iY, 3, new Integer(1), oPlot);
      oGrid.SetGridValue(iX, iY, 4, "test val 1", oPlot);
      oGrid.SetGridValue(iX, iY, 5, new Boolean(true), oPlot);

      oGrid.SetGridPackageValue(iX, iY, 0, 3, new Float(0.2), oPlot);
      oGrid.SetGridPackageValue(iX, iY, 0, 4, new Integer(2), oPlot);
      oGrid.SetGridPackageValue(iX, iY, 0, 5, "test val 2", oPlot);
      oGrid.SetGridPackageValue(iX, iY, 0, 6, new Boolean(true), oPlot);

      oVal = oGrid.GetGridValue(iX, iY, oPlot);
      assertEquals(0.1, oVal.mp_fFloats[0].floatValue(), 0.01);
      assertEquals(1, oVal.mp_iInts[0].intValue());
      assertEquals("test val 1", oVal.mp_sChars[0]);
      assertEquals(true, oVal.mp_bBools[0].booleanValue());

      oPackage = oGrid.GetPackageGridValue(oVal, 0);
      assertEquals(0.2, oPackage.mp_fFloats[0].floatValue(), 0.01);
      assertEquals(2, oPackage.mp_iInts[0].intValue());
      assertEquals("test val 2", oPackage.mp_sChars[0]);
      assertEquals(true, oPackage.mp_bBools[0].booleanValue());

      System.out.println("Value setting testing succeeded.");

    }
    catch (ModelException oErr) {
      fail("Value setting failed with message " + oErr.getMessage());
    }
  }

  /**
   * Tests setting the grid cell lengths.
   */
  public void testSetCellLength() {
    try {
      float fXCellLength = 1;
      Grid oGrid = new Grid(null, null, null, fXCellLength, fXCellLength);

      assertEquals(fXCellLength, oGrid.m_fLengthXCells, 0.0001);
      assertEquals(fXCellLength, oGrid.m_fLengthYCells, 0.0001);

      assertTrue(oGrid.m_bEdited == false);
      oGrid.SetXCellLength(fXCellLength);
      assertTrue(oGrid.m_bEdited == false);
      oGrid.SetYCellLength(fXCellLength);
      assertTrue(oGrid.m_bEdited == false);

      float fYCellLength = 3;
      oGrid.SetYCellLength(fYCellLength);
      assertTrue(oGrid.m_bEdited == true);
      assertEquals(fXCellLength, oGrid.m_fLengthXCells, 0.0001);
      assertEquals(fYCellLength, oGrid.m_fLengthYCells, 0.0001);

      oGrid.SetXCellLength(0);
      oGrid.SetYCellLength(0);
      oGrid.SetYCellLength(fYCellLength);
      assertEquals(fYCellLength, oGrid.m_fLengthXCells, 0.0001);
      assertEquals(fYCellLength, oGrid.m_fLengthYCells, 0.0001);

      System.out.println(
          "Grid cell length setting normal processing testing succeeded.");

    }
    catch (ModelException oErr) {
      fail("Constructor testing failed with message " + oErr.getMessage());
    }

    //Exception processing
    try {
      Grid oGrid = new Grid(null, null, null, 10, 10);
      oGrid.SetYCellLength( -14);
      fail("Did not catch negative value for grid cell length setting.");
    }
    catch (ModelException oErr) {
      ;
    }
    try {
      Grid oGrid = new Grid(null, null, null, 10, 10);
      oGrid.SetXCellLength( -14);
      fail("Did not catch negative value for grid cell length setting.");
    }
    catch (ModelException oErr) {
      ;
    }
  }

  /**
   * Tests XML grid map reading.
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

   */
  public void testGridMapReading() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      sFileName = WriteXMLGridMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      assertEquals(3, oManager.GetTreePopulation().GetTrees().size());

      Grid oGrid = oManager.GetGridByName("Quadrat GLI");
      assertFalse(oGrid == null);
      assertEquals(2, oGrid.mp_oGridVals.size());
      GridValue oVal = (GridValue) oGrid.mp_oGridVals.elementAt(0);
      assertEquals(0, oVal.GetCell().GetX());
      assertEquals(2, oVal.GetCell().GetY());
      assertEquals(12.65, oVal.mp_fFloats[0].floatValue(), 0.0001);
      assertEquals(0, oVal.mp_bBools.length);
      assertEquals(0, oVal.mp_iInts.length);
      assertEquals(0, oVal.mp_sChars.length);

      oVal = (GridValue) oGrid.mp_oGridVals.elementAt(1);
      assertEquals(3, oVal.GetCell().GetX());
      assertEquals(1, oVal.GetCell().GetY());
      assertEquals(45.22, oVal.mp_fFloats[0].floatValue(), 0.0001);
      assertEquals(0, oVal.mp_bBools.length);
      assertEquals(0, oVal.mp_iInts.length);
      assertEquals(0, oVal.mp_sChars.length);

      new File(sFileName).delete();

      //Now load a grid map
      sFileName = WriteXMLGridMap2();
      oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      oGrid = oManager.GetGridByName("Quadrat GLI");
      assertFalse(oGrid == null);
      assertEquals(4, oGrid.mp_oGridVals.size());
      oVal = (GridValue) oGrid.mp_oGridVals.elementAt(2);
      assertEquals(10, oVal.GetCell().GetX());
      assertEquals(11, oVal.GetCell().GetY());
      assertEquals(12.65, oVal.mp_fFloats[0].floatValue(), 0.0001);
      assertEquals(0, oVal.mp_bBools.length);
      assertEquals(0, oVal.mp_iInts.length);
      assertEquals(0, oVal.mp_sChars.length);

      oVal = (GridValue) oGrid.mp_oGridVals.elementAt(3);
      assertEquals(4, oVal.GetCell().GetX());
      assertEquals(9, oVal.GetCell().GetY());
      assertEquals(45.22, oVal.mp_fFloats[0].floatValue(), 0.0001);
      assertEquals(0, oVal.mp_bBools.length);
      assertEquals(0, oVal.mp_iInts.length);
      assertEquals(0, oVal.mp_sChars.length);

      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("XML tree map reading failed with message " + oErr.getMessage());
    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - bad X cell length
    //We don't validate X and Y cell length anymore because I haven't taken the
    //trouble to distinguish between a user-set change and a grid map-based
    //change
/*    try {
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteXMLGridMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      new File(sFileName).delete();

      //Now add an error file
      sFileName = WriteErrorXMLGridMap1();
      oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("XML grid map reading - bad X cell length was not caught.");

    } catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Exception processing - bad Y cell length
    try {
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteXMLGridMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      new File(sFileName).delete();

      //Now add an error file
      sFileName = WriteErrorXMLGridMap2();
      oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("XML grid map reading - bad Y cell length was not caught.");

    } catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    } */

    //Exception processing - bad X plot length
    try {
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      sFileName = WriteXMLGridMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      new File(sFileName).delete();

      //Now add an error file
      sFileName = WriteErrorXMLGridMap3();
      oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("XML grid map reading - bad X plot length was not caught.");

    } catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    //Exception processing - bad Y plot length
    try {
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      sFileName = WriteXMLGridMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      new File(sFileName).delete();

      //Now add an error file
      sFileName = WriteErrorXMLGridMap4();
      oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("XML grid map reading - bad Y plot length was not caught.");

    } catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    System.out.println("XML grid map reading test succeeded.");
  }

  /**
  * Writes a grid map that has different X grid length from WriteXMLGridMap2.
  * @return File name of new file.
  * @throws ModelException if there is a problem writing the file.
*
* <br>Edit history:
* <br>------------------
* <br>April 28, 2004: Submitted in beta version (LEM)

  */
 protected String WriteErrorXMLGridMap1() throws ModelException {
   try {
     String sFileName = "\\testgridmap2.xml";
     FileWriter oOut = new FileWriter(sFileName);

     oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
     oOut.write("<timestepRundata fileCode=\"06010701\">");
     oOut.write("<rt_timestep>0</rt_timestep>");

     //A false GLI grid map
     oOut.write("<grid gridName=\"Quadrat GLI\">");
     oOut.write("<ma_floatCodes>");
     oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
     oOut.write("</ma_floatCodes>");
     oOut.write("<ma_plotLenX>200.000000</ma_plotLenX>");
     oOut.write("<ma_plotLenY>200.000000</ma_plotLenY>");
     oOut.write("<ma_lengthXCells>2.000000</ma_lengthXCells>");
     oOut.write("<ma_lengthYCells>10.000000</ma_lengthYCells>");
     oOut.write("<ma_v x=\"11\" y=\"12\">");
     oOut.write("<fl c=\"0\">12.65</fl>");
     oOut.write("</ma_v>");
     oOut.write("<ma_v x=\"5\" y=\"10\">");
     oOut.write("<fl c=\"0\">45.22</fl>");
     oOut.write("</ma_v>");
     oOut.write("</grid>");
     oOut.write("</timestepRundata>");

     oOut.close();
     return sFileName;
   }
   catch (java.io.IOException oE) {
     throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
   }
 }

 /**
  * Writes a grid map that has different Y grid length from WriteXMLGridMap2.
  * @return File name of new file.
  * @throws ModelException if there is a problem writing the file.
*
* <br>Edit history:
* <br>------------------
* <br>April 28, 2004: Submitted in beta version (LEM)

  */
 protected String WriteErrorXMLGridMap2() throws ModelException {
   try {
     String sFileName = "\\testgridmap2.xml";
     FileWriter oOut = new FileWriter(sFileName);

     oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
     oOut.write("<timestepRundata fileCode=\"06010701\">");
     oOut.write("<rt_timestep>0</rt_timestep>");

     //A false GLI grid map
     oOut.write("<grid gridName=\"Quadrat GLI\">");
     oOut.write("<ma_floatCodes>");
     oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
     oOut.write("</ma_floatCodes>");
     oOut.write("<ma_plotLenX>200.000000</ma_plotLenX>");
     oOut.write("<ma_plotLenY>200.000000</ma_plotLenY>");
     oOut.write("<ma_lengthXCells>10.000000</ma_lengthXCells>");
     oOut.write("<ma_lengthYCells>2.000000</ma_lengthYCells>");
     oOut.write("<ma_v x=\"12\" y=\"13\">");
     oOut.write("<fl c=\"0\">12.65</fl>");
     oOut.write("</ma_v>");
     oOut.write("<ma_v x=\"6\" y=\"11\">");
     oOut.write("<fl c=\"0\">45.22</fl>");
     oOut.write("</ma_v>");
     oOut.write("</grid>");
     oOut.write("</timestepRundata>");

     oOut.close();
     return sFileName;
   }
   catch (java.io.IOException oE) {
     throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
   }
 }

 /**
  * Writes a grid map that has bad X plot length from WriteXMLGridMap2.
  * @return File name of new file.
  * @throws ModelException if there is a problem writing the file.
*
* <br>Edit history:
* <br>------------------
* <br>April 28, 2004: Submitted in beta version (LEM)

  */
 protected String WriteErrorXMLGridMap3() throws ModelException {
   try {
     String sFileName = "\\testgridmap2.xml";
     FileWriter oOut = new FileWriter(sFileName);

     oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
     oOut.write("<timestepRundata fileCode=\"06010701\">");
     oOut.write("<rt_timestep>0</rt_timestep>");

     //A false GLI grid map
     oOut.write("<grid gridName=\"Quadrat GLI\">");
     oOut.write("<ma_floatCodes>");
     oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
     oOut.write("</ma_floatCodes>");
     oOut.write("<ma_plotLenX>210.000000</ma_plotLenX>");
     oOut.write("<ma_plotLenY>200.000000</ma_plotLenY>");
     oOut.write("<ma_lengthXCells>10.000000</ma_lengthXCells>");
     oOut.write("<ma_lengthYCells>10.000000</ma_lengthYCells>");
     oOut.write("<ma_v x=\"13\" y=\"14\">");
     oOut.write("<fl c=\"0\">12.65</fl>");
     oOut.write("</ma_v>");
     oOut.write("<ma_v x=\"6\" y=\"11\">");
     oOut.write("<fl c=\"0\">45.22</fl>");
     oOut.write("</ma_v>");
     oOut.write("</grid>");
     oOut.write("</timestepRundata>");

     oOut.close();
     return sFileName;
   }
   catch (java.io.IOException oE) {
     throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
   }
 }

 /**
    * Writes a grid map that has bad Y plot length from WriteXMLGridMap2.
    * @return File name of new file.
    * @throws ModelException if there is a problem writing the file.
*
* <br>Edit history:
* <br>------------------
* <br>April 28, 2004: Submitted in beta version (LEM)

    */
   protected String WriteErrorXMLGridMap4() throws ModelException {
     try {
       String sFileName = "\\testgridmap2.xml";
       FileWriter oOut = new FileWriter(sFileName);

       oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
       oOut.write("<timestepRundata fileCode=\"06010701\">");
       oOut.write("<rt_timestep>0</rt_timestep>");

       //A false GLI grid map
       oOut.write("<grid gridName=\"Quadrat GLI\">");
       oOut.write("<ma_floatCodes>");
       oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
       oOut.write("</ma_floatCodes>");
       oOut.write("<ma_plotLenX>200.000000</ma_plotLenX>");
       oOut.write("<ma_plotLenY>220.000000</ma_plotLenY>");
       oOut.write("<ma_lengthXCells>10.000000</ma_lengthXCells>");
       oOut.write("<ma_lengthYCells>10.000000</ma_lengthYCells>");
       oOut.write("<ma_v x=\"13\" y=\"14\">");
       oOut.write("<fl c=\"0\">12.65</fl>");
       oOut.write("</ma_v>");
       oOut.write("<ma_v x=\"6\" y=\"11\">");
       oOut.write("<fl c=\"0\">45.22</fl>");
       oOut.write("</ma_v>");
       oOut.write("</grid>");
       oOut.write("</timestepRundata>");

       oOut.close();
       return sFileName;
     }
     catch (java.io.IOException oE) {
       throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
     }
   }

   /**
   * Writes a tree map file.  There are all four kinds of data members.  This
   * is actually a parameter file with a grid map in it.
   * @return The file name.
   * @throws ModelException
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

   */
  protected String WriteXMLGridMap1() throws ModelException {
    try {
      String sFileName = "\\testgridmap1.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<paramFile fileCode=\"06010101\">");

      //Plot
      oOut.write("<plot>");
      oOut.write("<plot_lenX>200.0</plot_lenX>");
      oOut.write("<plot_lenY>200.0</plot_lenY>");
      oOut.write("<plot_latitude>55.37</plot_latitude>");
      oOut.write("</plot>");

      //Trees and a tree map - to make sure it doesn't interfere with the grid
      //map reading
      oOut.write("<trees>");
      oOut.write("<tr_speciesList>");
      oOut.write("<tr_species speciesName=\"ACSA\"/>");
      oOut.write("<tr_species speciesName=\"ACRU\"/>");
      oOut.write("<tr_species speciesName=\"QURU\"/>");
      oOut.write("<tr_species speciesName=\"PRSE\"/>");
      oOut.write("<tr_species speciesName=\"PIST\"/>");
      oOut.write("<tr_species speciesName=\"TSCA\"/>");
      oOut.write("<tr_species speciesName=\"FRAM\"/>");
      oOut.write("<tr_species speciesName=\"BEAL\"/>");
      oOut.write("<tr_species speciesName=\"FAGR\"/>");
      oOut.write("</tr_speciesList>");
      oOut.write("<tr_treemap>");
      oOut.write("<tm_treeSettings sp=\"ACSA\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tree sp=\"0\" tp=\"2\">");
      oOut.write("<fl c=\"0\">6.34029</fl>");
      oOut.write("<fl c=\"1\">7.07883</fl>");
      oOut.write("<fl c=\"2\">15.7698</fl>");
      oOut.write("</tree>");
      oOut.write("<tree sp=\"0\" tp=\"2\">");
      oOut.write("<fl c=\"0\">7.34029</fl>");
      oOut.write("<fl c=\"1\">7.07883</fl>");
      oOut.write("<fl c=\"2\">15.7698</fl>");
      oOut.write("</tree>");
      oOut.write("<tree sp=\"0\" tp=\"2\">");
      oOut.write("<fl c=\"0\">8.34029</fl>");
      oOut.write("<fl c=\"1\">7.07883</fl>");
      oOut.write("<fl c=\"2\">15.7698</fl>");
      oOut.write("</tree>");
      oOut.write("</tr_treemap>");
      oOut.write("</trees>");

      //A false GLI grid map
      oOut.write("<grid gridName=\"Quadrat GLI\">");
      oOut.write("<ma_floatCodes>");
      oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
      oOut.write("</ma_floatCodes>");
      oOut.write("<ma_plotLenX>200.000000</ma_plotLenX>");
      oOut.write("<ma_plotLenY>200.000000</ma_plotLenY>");
      oOut.write("<ma_lengthXCells>10.000000</ma_lengthXCells>");
      oOut.write("<ma_lengthYCells>10.000000</ma_lengthYCells>");
      oOut.write("<ma_v x=\"0\" y=\"2\">");
      oOut.write("<fl c=\"0\">12.65</fl>");
      oOut.write("</ma_v>");
      oOut.write("<ma_v x=\"3\" y=\"1\">");
      oOut.write("<fl c=\"0\">45.22</fl>");
      oOut.write("</ma_v>");
      oOut.write("</grid>");
      oOut.write("</paramFile>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
    }
  }

  /**
   * This writes a fake rundata file with a grid map in it.
   * @return File name of new file.
   * @throws ModelException if there is a problem writing the file.
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

   */
  protected String WriteXMLGridMap2() throws ModelException {
    try {
      String sFileName = "\\testgridmap2.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<timestepRundata fileCode=\"06010701\">");
      oOut.write("<rt_timestep>0</rt_timestep>");

      //A false GLI grid map
      oOut.write("<grid gridName=\"Quadrat GLI\">");
      oOut.write("<ma_floatCodes>");
      oOut.write("<ma_floatCode label=\"GLI\">2</ma_floatCode>");
      oOut.write("</ma_floatCodes>");
      oOut.write("<ma_plotLenX>200.000000</ma_plotLenX>");
      oOut.write("<ma_plotLenY>200.000000</ma_plotLenY>");
      oOut.write("<ma_lengthXCells>10.000000</ma_lengthXCells>");
      oOut.write("<ma_lengthYCells>10.000000</ma_lengthYCells>");
      oOut.write("<ma_v x=\"10\" y=\"11\">");
      oOut.write("<fl c=\"2\">12.65</fl>");
      oOut.write("</ma_v>");
      oOut.write("<ma_v x=\"4\" y=\"9\">");
      oOut.write("<fl c=\"2\">45.22</fl>");
      oOut.write("</ma_v>");
      oOut.write("</grid>");
      oOut.write("</timestepRundata>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
    }
  }
}
