package javawrapper;


import java.io.FileWriter;
import java.io.File;

/**
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestSeedPredationBehaviors
    extends ModelTestCase {

  /**
   * Tests that the correct parameters are displayed for each behavior.
   */
  public void testFormatDataForDisplay() {
    try {
      GUIManager oManager = new GUIManager(null);
      SeedPredationBehaviors oPredation = null;
      TreePopulation oPop = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no seed predation behaviors are enabled.
      oManager.InputXMLParameterFile(WriteNoSeedPredationXMLFile());
      oPredation = oManager.GetSeedPredationBehaviors();
      oPop = oManager.GetTreePopulation();
      assertEquals(null, oPredation.FormatDataForDisplay(oPop));

      //Case:  only functional response seed predation is enabled.
      oManager.InputXMLParameterFile(WriteNoSeedPredationXMLFile());
      oPredation = oManager.GetSeedPredationBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oPredation.GetBehaviorByKey("funcResponsePredation");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          oPop));
      p_sExpected = new String[] {
          "Func Resp - Predator Initial Density - num/sq m",
          "Func Resp - Max Decline Rate, Season 1, predators/week",
          "Func Resp - Max Decline Rate, Season 2, predators/week",
          "Func Resp - Demographic Efficiency, Season 1",
          "Func Resp - Demographic Efficiency, Season 2",
          "Func Resp - Density Dependent Coeff, Season 1",
          "Func Resp - Density Dependent Coeff, Season 2",
          "Func Resp - Max Intake Rate - seeds per predator per day",
          "Func Resp - Foraging Efficiency",
          "Func Resp - Proportion of Seeds Germinating Each Week",
          "Func Resp - Number of Weeks in Which Seedfall Occurs",
          "Func Resp - Weeks to Run Seed Predation Model (1 - 52)",
          "Func Resp - Week Germination Begins",
          "Func Resp - Week Season 2 Begins",
          "Func Resp - Seed Predation Output Filename, If Desired",
          "Func Resp - Keep Predator Densities Between Timesteps",
      };
      TestTable(oPredation.FormatDataForDisplay(oPop), p_sExpected);
            
      //Case:  only neighborhood seed predation is enabled
      oManager.InputXMLParameterFile(WriteNoSeedPredationXMLFile());
      oPredation = oManager.GetSeedPredationBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oPredation.GetBehaviorByKey("Neighborhood Predation");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          oPop));
      p_sExpected = new String[] {
          "Neighborhood Predation - Masting \"p0\"",
          "Neighborhood Predation - Non-Masting \"p0\"",
          "Neighborhood Predation - Species 1 Masting \"pn\"",
          "Neighborhood Predation - Species 1 Non-Masting \"pn\"",
          "Neighborhood Predation - Masting Seed Density, #/m2/yr",
          "Neighborhood Predation - Minimum Neighbor DBH (cm)",
          "Neighborhood Predation - Counts For Masting?",
          "Neighborhood Predation - Neighbor Search Radius (m)", 
          "Neighborhood Predation - Mast Event Decision Method",
      };
      TestTable(oPredation.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: linked functional response seed predation is enabled.
      oManager.InputXMLParameterFile(WriteNoSeedPredationXMLFile());
      oPredation = oManager.GetSeedPredationBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oPredation.GetBehaviorByKey("Linked func response predation");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          oPop));
      p_sExpected = new String[] {
          "(Lnk) Func Resp - Predator Initial Density - num/sq m",
          "(Lnk) Func Resp - Max Decline Rate, Season 1, predators/week",
          "(Lnk) Func Resp - Max Decline Rate, Season 2, predators/week",
          "(Lnk) Func Resp - Demographic Efficiency, Season 1",
          "(Lnk) Func Resp - Demographic Efficiency, Season 2",
          "(Lnk) Func Resp - Density Dependent Coeff, Season 1",
          "(Lnk) Func Resp - Density Dependent Coeff, Season 2",
          "(Lnk) Func Resp - Max Intake Rate - seeds per predator per day",
          "(Lnk) Func Resp - Foraging Efficiency",
          "(Lnk) Func Resp - Proportion of Seeds Germinating Each Week",
          "(Lnk) Func Resp - Number of Weeks in Which Seedfall Occurs",
          "(Lnk) Func Resp - Weeks to Run Seed Predation Model (1 - 52)",
          "(Lnk) Func Resp - Week Germination Begins",
          "(Lnk) Func Resp - Week Season 2 Begins",
          "(Lnk) Func Resp - Seed Predation Output Filename, If Desired",
          "(Lnk) Func Resp - Keep Predator Densities Between Timesteps",
      };
      TestTable(oPredation.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case:  only linked neighborhood seed predation is enabled
      oManager.InputXMLParameterFile(WriteNoSeedPredationXMLFile());
      oPredation = oManager.GetSeedPredationBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oPredation.GetBehaviorByKey("Linked Neighborhood Predation");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          oPop));
      p_sExpected = new String[] {
          "(Lnk) Neighborhood Predation - \"p0\"",
          "(Lnk) Neighborhood Predation - Species 1 \"pn\"",
          "(Lnk) Neighborhood Predation - Minimum Neighbor DBH (cm)",
          "(Lnk) Neighborhood Predation - Neighbor Search Radius (m)",    
      };
      TestTable(oPredation.FormatDataForDisplay(oPop), p_sExpected);
       
      //Case:  all are enabled.
      oManager.InputXMLParameterFile(WriteNoSeedPredationXMLFile());
      oPredation = oManager.GetSeedPredationBehaviors();
      oPop = oManager.GetTreePopulation();
      int i;
      for (i = 0; i < oPredation.mp_oChildBehaviors.length; i++) {
        oPredation.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oPredation.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
            SpeciesTypeCombo(0, TreePopulation.SEED,
                             oPop));
      }
      p_sExpected = new String[] {
          "Func Resp - Predator Initial Density - num/sq m",
          "Func Resp - Max Decline Rate, Season 1, predators/week",
          "Func Resp - Max Decline Rate, Season 2, predators/week",
          "Func Resp - Demographic Efficiency, Season 1",
          "Func Resp - Demographic Efficiency, Season 2",
          "Func Resp - Density Dependent Coeff, Season 1",
          "Func Resp - Density Dependent Coeff, Season 2",
          "Func Resp - Max Intake Rate - seeds per predator per day",
          "Func Resp - Foraging Efficiency",
          "Func Resp - Proportion of Seeds Germinating Each Week",
          "Func Resp - Number of Weeks in Which Seedfall Occurs",
          "Func Resp - Weeks to Run Seed Predation Model (1 - 52)",
          "Func Resp - Week Germination Begins",
          "Func Resp - Week Season 2 Begins",
          "Func Resp - Seed Predation Output Filename, If Desired",
          "Func Resp - Keep Predator Densities Between Timesteps",
          "Neighborhood Predation - Masting \"p0\"",
          "Neighborhood Predation - Non-Masting \"p0\"",
          "Neighborhood Predation - Species 1 Masting \"pn\"",
          "Neighborhood Predation - Species 1 Non-Masting \"pn\"",
          "Neighborhood Predation - Masting Seed Density, #/m2/yr",
          "Neighborhood Predation - Minimum Neighbor DBH (cm)",
          "Neighborhood Predation - Counts For Masting?",
          "Neighborhood Predation - Neighbor Search Radius (m)", 
          "Neighborhood Predation - Mast Event Decision Method",
          "(Lnk) Func Resp - Predator Initial Density - num/sq m",
          "(Lnk) Func Resp - Max Decline Rate, Season 1, predators/week",
          "(Lnk) Func Resp - Max Decline Rate, Season 2, predators/week",
          "(Lnk) Func Resp - Demographic Efficiency, Season 1",
          "(Lnk) Func Resp - Demographic Efficiency, Season 2",
          "(Lnk) Func Resp - Density Dependent Coeff, Season 1",
          "(Lnk) Func Resp - Density Dependent Coeff, Season 2",
          "(Lnk) Func Resp - Max Intake Rate - seeds per predator per day",
          "(Lnk) Func Resp - Foraging Efficiency",
          "(Lnk) Func Resp - Proportion of Seeds Germinating Each Week",
          "(Lnk) Func Resp - Number of Weeks in Which Seedfall Occurs",
          "(Lnk) Func Resp - Weeks to Run Seed Predation Model (1 - 52)",
          "(Lnk) Func Resp - Week Germination Begins",
          "(Lnk) Func Resp - Week Season 2 Begins",
          "(Lnk) Func Resp - Seed Predation Output Filename, If Desired",
          "(Lnk) Func Resp - Keep Predator Densities Between Timesteps",
          "(Lnk) Neighborhood Predation - \"p0\"",
          "(Lnk) Neighborhood Predation - Species 1 \"pn\"",
          "(Lnk) Neighborhood Predation - Minimum Neighbor DBH (cm)",
          "(Lnk) Neighborhood Predation - Neighbor Search Radius (m)",
      };
      TestTable(oPredation.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for seed predation.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Makes sure all parameters read correctly.
   */
  public void testParFileReading() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);
      ModelData oData;
      ModelVector oVector;

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      SeedPredationBehaviors oPred = oManager.GetSeedPredationBehaviors();
      double SMALL_VAL = 0.00001;
      int i, iIndex;
      
      assertEquals(oPred.m_fFuncRespPredInitDensity.GetValue(), 0.46875, SMALL_VAL);
      assertEquals(oPred.m_fFuncRespMaxInstDeclineRate1.GetValue(), -0.050622442, SMALL_VAL);
      assertEquals(oPred.m_fFuncRespMaxInstDeclineRate2.GetValue(), -1.050622442, SMALL_VAL);
      assertEquals(oPred.m_fFuncRespDemogEfficiency1.GetValue(), 0.154258, SMALL_VAL);
      assertEquals(oPred.m_fFuncRespDemogEfficiency2.GetValue(), 2.154258, SMALL_VAL);
      assertEquals(oPred.m_fFunRespDensDepCoeff1.GetValue(), -0.718667, SMALL_VAL);
      assertEquals(oPred.m_fFunRespDensDepCoeff2.GetValue(), -0.818667, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fFuncRespMaxIntake.GetValue().elementAt(0)).
          floatValue(), 1042.2, 0.1);
      assertEquals( ( (Float) oPred.mp_fFuncRespMaxIntake.GetValue().elementAt(1)).
          floatValue(), 800.2, 0.1);
      assertEquals( ( (Float) oPred.mp_fFuncRespForagingEff.GetValue().elementAt(0)).
          floatValue(), 0.001389, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fFuncRespForagingEff.GetValue().elementAt(1)).
          floatValue(), 0.3389, SMALL_VAL);
      assertEquals(oPred.m_iFuncRespNumWeeksSeedFall.GetValue(), 12);
      assertEquals(oPred.m_iFuncRespWeekSeason2Starts.GetValue(), 20);
      assertEquals(oPred.m_iFuncRespNumWeeksToModel.GetValue(), 52);
      assertEquals(oPred.m_iFuncRespWeekGermStarts.GetValue(), 32);
      assertEquals(oPred.m_iPreservePredatorDensities.GetValue(), 0);
      assertEquals(oPred.m_fFuncRespPropGerm.GetValue(), 0.35, SMALL_VAL);
      assertEquals(oPred.m_sFuncRespOutputFilename.GetValue(), "TestSeeds.txt");
      
      assertEquals(oPred.m_fLnkFuncRespPredInitDensity.GetValue(), 0.002, SMALL_VAL);
      assertEquals(oPred.m_fLnkFuncRespMaxInstDeclineRate1.GetValue(), -0.04, SMALL_VAL);
      assertEquals(oPred.m_fLnkFuncRespMaxInstDeclineRate2.GetValue(), -1.04, SMALL_VAL);
      assertEquals(oPred.m_fLnkFuncRespDemogEfficiency1.GetValue(), 0.0002, SMALL_VAL);
      assertEquals(oPred.m_fLnkFuncRespDemogEfficiency2.GetValue(), 0.3, SMALL_VAL);
      assertEquals(oPred.m_fLnkFunRespDensDepCoeff1.GetValue(), -0.008, SMALL_VAL);
      assertEquals(oPred.m_fLnkFunRespDensDepCoeff2.GetValue(), -0.8, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fLnkFuncRespMaxIntake.GetValue().elementAt(0)).
          floatValue(), 300, 0.1);
      assertEquals( ( (Float) oPred.mp_fLnkFuncRespMaxIntake.GetValue().elementAt(1)).
          floatValue(), 245, 0.1);
      assertEquals( ( (Float) oPred.mp_fLnkFuncRespForagingEff.GetValue().elementAt(0)).
          floatValue(), 0.002, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fLnkFuncRespForagingEff.GetValue().elementAt(1)).
          floatValue(), 0.444, SMALL_VAL);
      assertEquals(oPred.m_iLnkFuncRespNumWeeksSeedFall.GetValue(), 7);
      assertEquals(oPred.m_iLnkFuncRespNumWeeksToModel.GetValue(), 30);
      assertEquals(oPred.m_iLnkFuncRespWeekSeason2Starts.GetValue(), 21);
      assertEquals(oPred.m_iLnkFuncRespWeekGermStarts.GetValue(), 10);
      assertEquals(oPred.m_iLnkPreservePredatorDensities.GetValue(), 1);
      assertEquals(oPred.m_fLnkFuncRespPropGerm.GetValue(), 0.1, SMALL_VAL);
      assertEquals(oPred.m_sLnkFuncRespOutputFilename.GetValue(), "TestLinkSeeds.txt");
      
      assertEquals(oPred.m_fNeighPredMastingDensity.GetValue(), 4, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredMinDbh.GetValue().elementAt(0)).
          floatValue(), 10, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMinDbh.GetValue().elementAt(1)).
          floatValue(), 15, SMALL_VAL);
      
      ModelEnum oEnum;
      oEnum = (ModelEnum) oPred.mp_iNeighPredCounts4Mast.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oPred.mp_iNeighPredCounts4Mast.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 0);
      
      assertEquals(oPred.m_fNeighPredRadius.GetValue(), 15, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(0)).
          floatValue(), 1.32, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(1)).
          floatValue(), 0.86, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(0)).
          floatValue(), -2.51, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(1)).
          floatValue(), -1.36, SMALL_VAL);
      
      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Masting \"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) -5.94, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0, SMALL_VAL);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex + 1);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 2.8, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 3.1, SMALL_VAL);
      
      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Non-Masting \"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 2.56, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.23, SMALL_VAL);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex + 1);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 4.91, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 1.9, SMALL_VAL);
      
      
      assertEquals( ( (Float) oPred.mp_fLnkNeighPredMinDbh.GetValue().elementAt(0)).
          floatValue(), 13, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fLnkNeighPredMinDbh.GetValue().elementAt(1)).
          floatValue(), 14, SMALL_VAL);
      
      assertEquals(oPred.m_fLnkNeighPredRadius.GetValue(), 11, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fLnkNeighPredP0.GetValue().elementAt(0)).
          floatValue(), 1.67, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fLnkNeighPredP0.GetValue().elementAt(1)).
          floatValue(), 1.83, SMALL_VAL);
      
      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf("(Lnk)") > -1 &&
            oData.GetDescriptor().indexOf("\"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) -5.32, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) -3.2, SMALL_VAL);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex + 1);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 2.1, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 3.4, SMALL_VAL);
    }
    catch (ModelException oErr) {
      fail("Seed predation validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
    	new File(sFileName).delete();
    }
  }
    
  /**
   * Tests ValidateData().
   * @throws ModelException if testing fails
   */
  public void testValidateData() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    SeedPredationBehaviors oPred = null;
    try {
      try {

        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetSeedPredationBehaviors().ValidateData(oManager.
            GetTreePopulation());
      }
      catch (ModelException oErr) {
        fail("Seed predation validation failed with message " + oErr.getMessage());
      }

      //Exception processing - functional response predation is enabled but
      //no disperse behaviors are
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.GetBehaviorByKey("Neighborhood Predation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("Neighborhood Predation").ClearSpeciesTypeCombos();
        oPred.GetBehaviorByKey("Linked Neighborhood Predation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("Linked Neighborhood Predation").ClearSpeciesTypeCombos();
        oPred.GetBehaviorByKey("Linked func response predation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("Linked func response predation").ClearSpeciesTypeCombos();
        oManager.GetDisperseBehaviors().GetBehaviorByKey("non spatial disperse").
            m_bIsEnabled = false;
        oManager.GetDisperseBehaviors().GetBehaviorByKey("non spatial disperse").
            ClearSpeciesTypeCombos();
        oPred.ValidateData(oManager.GetTreePopulation());
        fail(
            "Seed predation didn't catch no disperse with functional response" +
            " predation.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - linked functional response predation is enabled 
      //but no disperse behaviors are
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.GetBehaviorByKey("Neighborhood Predation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("Neighborhood Predation").ClearSpeciesTypeCombos();
        oPred.GetBehaviorByKey("Linked Neighborhood Predation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("Linked Neighborhood Predation").ClearSpeciesTypeCombos();
        oPred.GetBehaviorByKey("funcResponsePredation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("funcResponsePredation").ClearSpeciesTypeCombos();
        oManager.GetDisperseBehaviors().GetBehaviorByKey("non spatial disperse").
            m_bIsEnabled = false;
        oManager.GetDisperseBehaviors().GetBehaviorByKey("non spatial disperse").
            ClearSpeciesTypeCombos();
        oPred.ValidateData(oManager.GetTreePopulation());
        fail(
            "Seed predation didn't catch no disperse with functional response" +
            " predation.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());

      //Exception processing - neighborhood predation is enabled but
      //no disperse behaviors are
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.GetBehaviorByKey("funcResponsePredation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("funcResponsePredation").ClearSpeciesTypeCombos();
        oPred.GetBehaviorByKey("Linked Neighborhood Predation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("Linked Neighborhood Predation").ClearSpeciesTypeCombos();
        oPred.GetBehaviorByKey("Linked func response predation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("Linked func response predation").ClearSpeciesTypeCombos();
        oManager.GetDisperseBehaviors().GetBehaviorByKey("non spatial disperse").
            m_bIsEnabled = false;
        oManager.GetDisperseBehaviors().GetBehaviorByKey("non spatial disperse").
            ClearSpeciesTypeCombos();
        oPred.ValidateData(oManager.GetTreePopulation());
        fail(
            "Seed predation didn't catch no disperse with functional response" +
            " predation.");
      }
      catch (ModelException oErr) {
        ;
      }
      
      //Exception processing - linked neighborhood predation is enabled but
      //not linked functional response predation
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.GetBehaviorByKey("funcResponsePredation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("funcResponsePredation").ClearSpeciesTypeCombos();
        oPred.GetBehaviorByKey("Neighborhood Predation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("Neighborhood Predation").ClearSpeciesTypeCombos();
        oPred.GetBehaviorByKey("Linked func response predation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("Linked func response predation").ClearSpeciesTypeCombos();
        oPred.ValidateData(oManager.GetTreePopulation());
        fail(
            "Seed predation didn't catch no linked functional response with " +
            "linked neighborhood predation.");
      }
      catch (ModelException oErr) {
        ;
      }
      
      //Exception processing - case: number of weeks of seedfall is less than
      //or equal to zero
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iFuncRespNumWeeksSeedFall.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for number of weeks of" +
        		"seedfall.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: number of weeks of seedfall is less than
      //or equal to zero (linked)
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iLnkFuncRespNumWeeksSeedFall.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for number of weeks of" +
            "seedfall.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Initial predator density is less than or equal to zero
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fFuncRespPredInitDensity.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for initial predator " +
        		"densities.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Initial predator density is less than or 
      //equal to zero (linked)
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fLnkFuncRespPredInitDensity.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for initial predator " +
            "densities.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Number of weeks to run the model is less than 0
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iFuncRespNumWeeksToModel.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for number of weeks to " +
        		"run the model.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Number of weeks to run the model is less 
      //than 0 (linked)
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iLnkFuncRespNumWeeksToModel.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for number of weeks to " +
            "run the model.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Number of weeks to run the model is greater than 52
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iFuncRespNumWeeksToModel.SetValue(63);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for number of weeks to " +
        		"run the model.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Number of weeks to run the model is 
      //greater than 52 (linked)
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iLnkFuncRespNumWeeksToModel.SetValue(63);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for number of weeks to " +
            "run the model.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Week to start germination is less than 0
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iFuncRespWeekGermStarts.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for week germination " +
        		"starts.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Week to start germination is less than 0
      //(linked)
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iLnkFuncRespWeekGermStarts.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for week germination " +
            "starts.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Week to start germination is greater than 52
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iFuncRespWeekGermStarts.SetValue(63);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for week germination " +
        		"starts.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Week to start germination is greater 
      //than 52 (linked)
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iLnkFuncRespWeekGermStarts.SetValue(63);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for week germination " +
            "starts.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Proportion germinating is not a proportion
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fFuncRespPropGerm.SetValue((float)1.3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for proportion germinating.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Proportion germinating is not a proportion
      //(linked)
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fLnkFuncRespPropGerm.SetValue((float)1.3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for proportion germinating.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Neighborhood predation minimum neighbor 
      //DBH is less than zero
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.mp_fNeighPredMinDbh.GetValue().remove(0);
        oPred.mp_fNeighPredMinDbh.GetValue().add(new Float(-1.3));
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for minimum neighbor DBH.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Neighborhood predation minimum neighbor 
      //DBH is less than zero (linked)
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.mp_fLnkNeighPredMinDbh.GetValue().remove(0);
        oPred.mp_fLnkNeighPredMinDbh.GetValue().add(new Float(-1.3));
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for minimum neighbor DBH.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Neighborhood predation neighbor radius is
      //less than zero
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fNeighPredRadius.SetValue((float)-1.3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for neighborhood radius.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Neighborhood predation neighbor radius is
      //less than zero (linked)
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fLnkNeighPredRadius.SetValue((float)-1.3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for neighborhood radius.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Neighborhood predation masting seed
      //density threshold is less than zero
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fNeighPredMastingDensity.SetValue((float)-1.3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for masting density" +
        		" threshold.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());  
      
    } catch (ModelException oErr) {
    	fail("Seed predation validation failed with message " + oErr.getMessage());
    } catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }
  
  /**
   * Makes sure that pn values are managed correctly when species change.
   */
  public void testChangeOfSpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      double SMALL_VAL = 0.00001;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      SeedPredationBehaviors oPred = oManager.GetSeedPredationBehaviors();
      
      //Count the total number of masting pns
      int iCount = 0;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Masting \"pn\"") > -1) {
          iCount++;
        }
      }
      assertEquals(2, iCount);
      
      //Count the total number of non-masting pns
      iCount = 0;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Non-Masting \"pn\"") > -1) {
          iCount++;
        }
      }
      assertEquals(2, iCount);
      
      //Count the total number of linked pns
      iCount = 0;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf("(Lnk)") > -1 &&
            oData.GetDescriptor().indexOf("\"pn\"") > -1) {
          iCount++;
        }
      }
      assertEquals(2, iCount);
      
      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Masting \"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) -5.94, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0, SMALL_VAL);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex + 1);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 2.8, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 3.1, SMALL_VAL);
      
      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Non-Masting \"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 2.56, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.23, SMALL_VAL);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex + 1);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 4.91, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 1.9, SMALL_VAL);

      //Change the species
      String[] sNewSpecies = new String[] {
            "Species 2",
            "Species 3",
            "Species 4"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      oPred = oManager.GetSeedPredationBehaviors();

      //Count the total number of masting pns
      iCount = 0;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Masting \"pn\"") > -1) {
          iCount++;
        }
      }
      assertEquals(3, iCount);

      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Masting \"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 3.1, SMALL_VAL);

      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 3") &&
                 -1 != oVector.GetDescriptor().indexOf(" Masting \"pn\""));

      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 4") &&
                 -1 != oVector.GetDescriptor().indexOf(" Masting \"pn\""));

      iIndex++;
      if (iIndex < oPred.mp_oAllData.size()) {
       oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
       assertTrue(-1 == oVector.GetDescriptor().indexOf(" Masting \"pn\""));
      }
      
      //Now do the non-masting pns
      iCount = 0;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Non-Masting \"pn\"") > -1) {
          iCount++;
        }
      }
      assertEquals(3, iCount);

      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Non-Masting \"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 1.9, SMALL_VAL);

      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 3") &&
                 -1 != oVector.GetDescriptor().indexOf(" Non-Masting \"pn\""));

      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 4") &&
                 -1 != oVector.GetDescriptor().indexOf(" Non-Masting \"pn\""));

      iIndex++;
      if (iIndex < oPred.mp_oAllData.size()) {
       oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
       assertTrue(-1 == oVector.GetDescriptor().indexOf(" Non-Masting \"pn\""));
     } 
      
     //Now do the linked pns
     iCount = 0;
     for (i = 0; i < oPred.mp_oAllData.size(); i++) {
       oData = (ModelData) oPred.mp_oAllData.elementAt(i);
       if (oData.GetDescriptor().indexOf("(Lnk)") > -1 &&
           oData.GetDescriptor().indexOf("\"pn\"") > -1) {
         iCount++;
       }
     }
     assertEquals(3, iCount);

     for (i = 0; i < oPred.mp_oAllData.size(); i++) {
      oData = (ModelData) oPred.mp_oAllData.elementAt(i);
      if (oData.GetDescriptor().indexOf("(Lnk)") > -1 &&
          oData.GetDescriptor().indexOf("\"pn\"") > -1) {
        iIndex = i;
        break;
      }
    }
    oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
    assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
    assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                 (float) 3.4, SMALL_VAL);

    iIndex++;
    oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
    assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 3") &&
               -1 != oVector.GetDescriptor().indexOf("(Lnk)") &&
               -1 != oVector.GetDescriptor().indexOf("\"pn\""));

    iIndex++;
    oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
    assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 4") &&
        -1 != oVector.GetDescriptor().indexOf("(Lnk)") &&
        -1 != oVector.GetDescriptor().indexOf("\"pn\""));

    iIndex++;
     if (iIndex < oPred.mp_oAllData.size()) {
       oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
       assertTrue(-1 == oVector.GetDescriptor().indexOf("(Lnk)") &&
           -1 == oVector.GetDescriptor().indexOf("\"pn\""));
     } 
    }
    catch (ModelException oErr) {
      fail("Seed predation validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Makes sure that pn values are managed correctly when species are copied.
   */
  public void testCopySpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      double SMALL_VAL = 0.00001;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLFileForSpeciesCopy();
      oManager.InputXMLParameterFile(sFileName);
      SeedPredationBehaviors oPred = oManager.GetSeedPredationBehaviors();
      
      Behavior oBeh = oPred.GetBehaviorByKey("Neighborhood Predation");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(4, oBeh.GetNumberOfCombos());
      SpeciesTypeCombo oCombo;
      for (i = 0; i < oBeh.GetNumberOfCombos(); i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(0, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }
      
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(0)).
          floatValue(), 0.01, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(1)).
          floatValue(), 0.02, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(2)).
          floatValue(), 0.03, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(3)).
          floatValue(), 0.04, SMALL_VAL);
      
      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Masting \"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.05, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.06, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
                   (float) 0.07, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
                   (float) 0.08, SMALL_VAL);
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.09, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.1, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.11, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.12, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.13, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.14, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.15, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.16, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.17, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.18, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.19, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.2, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.21, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.22, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.23, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.24, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.25, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.26, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.27, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.28, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.29, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.3, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.31, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.32, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.33, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.34, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.35, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.36, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.37, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.38, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.39, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.4, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(0)).
          floatValue(), 1.01, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(1)).
          floatValue(), 1.02, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(2)).
          floatValue(), 1.03, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(3)).
          floatValue(), 1.04, SMALL_VAL);
      
      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Non-Masting \"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 1.05, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 1.06, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
                   (float) 1.07, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
                   (float) 1.08, SMALL_VAL);
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.09, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.1, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.11, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.12, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.13, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.14, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.15, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.16, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.17, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.18, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.19, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.2, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.21, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.22, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.23, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.24, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.25, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.26, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.27, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.28, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.29, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.3, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.31, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.32, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.33, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.34, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.35, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.36, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.37, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.38, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.39, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.4, SMALL_VAL);
      
      
      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf("(Lnk)") > -1 &&
            oData.GetDescriptor().indexOf("\"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 1.41, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 1.42, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
                   (float) 1.43, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
                   (float) 1.44, SMALL_VAL);
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.45, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.46, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.47, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.48, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.49, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.5, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.51, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.52, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.53, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.54, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.55, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.56, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.57, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.58, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.59, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.6, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.61, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.62, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.63, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.64, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.65, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.66, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.67, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.68, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.69, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.7, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.71, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.72, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.73, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.74, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.75, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.76, SMALL_VAL);

      //Copy the species
      oManager.GetTreePopulation().DoCopySpecies("Western_Redcedar", "Black_Cottonwood");

      oBeh = oPred.GetBehaviorByKey("Neighborhood Predation");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(5, oBeh.GetNumberOfCombos());
      for (i = 0; i < oBeh.GetNumberOfCombos() - 1; i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(0, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }
      oCombo = oBeh.GetSpeciesTypeCombo(4);
      assertEquals(0, oCombo.GetType());
      assertEquals(7, oCombo.GetSpecies());

      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(0)).
          floatValue(), 0.01, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(1)).
          floatValue(), 0.02, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(2)).
          floatValue(), 0.03, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(3)).
          floatValue(), 0.04, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingP0.GetValue().elementAt(7)).
          floatValue(), 0.02, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(0)).
          floatValue(), 1.01, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(1)).
          floatValue(), 1.02, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(2)).
          floatValue(), 1.03, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(3)).
          floatValue(), 1.04, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingP0.GetValue().elementAt(7)).
          floatValue(), 1.02, SMALL_VAL);

      //Pns
      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Masting \"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.05, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.06, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.07, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.08, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 0.06, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.09, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.1, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.11, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.12, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 0.1, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.13, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.14, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.15, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.16, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 0.14, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.17, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.18, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.19, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.2, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 0.18, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.21, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.22, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.23, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.24, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 0.22, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.25, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.26, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.27, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.28, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 0.26, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.29, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.3, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.31, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.32, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 0.3, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.09, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.1, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.11, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.12, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 0.1, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 0.37, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 0.38, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 0.39, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 0.4, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 0.38, SMALL_VAL);


      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf(" Non-Masting \"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.05, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.06, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.07, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.08, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.06, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.09, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.1, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.11, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.12, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.1, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.13, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.14, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.15, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.16, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.14, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.17, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.18, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.19, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.2, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.18, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.21, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.22, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.23, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.24, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.22, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.25, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.26, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.27, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.28, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.26, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.29, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.3, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.31, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.32, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.3, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.09, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.1, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.11, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.12, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.1, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.37, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.38, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.39, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.4, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.38, SMALL_VAL);
      
      //Linked pn
      iIndex = -1;
      for (i = 0; i < oPred.mp_oAllData.size(); i++) {
        oData = (ModelData) oPred.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().indexOf("(Lnk)") > -1 &&
            oData.GetDescriptor().indexOf("\"pn\"") > -1) {
          iIndex = i;
          break;
        }
      }
      assertTrue(iIndex >= 0);
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 1.41, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 1.42, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
                   (float) 1.43, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
                   (float) 1.44, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.42, SMALL_VAL);
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.45, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.46, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.47, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.48, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.46, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.49, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.5, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.51, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.52, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.5, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.53, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.54, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.55, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.56, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.54, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.57, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.58, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.59, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.6, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.58, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.61, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.62, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.63, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.64, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.62, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.65, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.66, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.67, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.68, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.66, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.45, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.46, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.47, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.48, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.46, SMALL_VAL);
      
      iIndex++;
      oVector = (ModelVector) oPred.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
          (float) 1.73, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
          (float) 1.74, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(2)).floatValue(),
          (float) 1.75, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(3)).floatValue(),
          (float) 1.76, SMALL_VAL);
      assertEquals( ( (Number) oVector.GetValue().elementAt(7)).floatValue(),
          (float) 1.74, SMALL_VAL);
     

      System.out.println("Test copy species was successful.");
    }
    catch (ModelException oErr) {
      fail("Test copy species failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }
    
  /**
   * Writes a valid seed predation parameter file with all the behaviors.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  private String WriteValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">"); 
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>96</plot_lenX>");
    oOut.write("<plot_lenY>96</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\"/>");
    oOut.write("<tr_species speciesName=\"Species_2\"/>");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_maxSeedlingHeight>");
    oOut.write("<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>");
    oOut.write("</tr_maxSeedlingHeight>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.54</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0614</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_interceptDiam10ToDBH>");
    oOut.write("<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>");
    oOut.write("</tr_interceptDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.368</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("<tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>");
    oOut.write("</tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_whatSaplingHeightDiam>");
    oOut.write("<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>");
    oOut.write("</tr_whatSaplingHeightDiam>");
    oOut.write("<tr_whatAdultHeightDiam>");
    oOut.write("<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>");
    oOut.write("</tr_whatAdultHeightDiam>");
    oOut.write("<tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>");
    oOut.write("</tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>");
    oOut.write("</tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>");
    oOut.write("</tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_whatSaplingCrownHeightHeight>");
    oOut.write("<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>");
    oOut.write("</tr_whatSaplingCrownHeightHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>non spatial disperse</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>"); 
    oOut.write("<behaviorName>functional response seed predation</behaviorName>");
    oOut.write("<version>1</version>"); 
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>"); 
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Neighborhood Seed Predation</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>"); 
    oOut.write("<behaviorName>linked functional response seed predation</behaviorName>");
    oOut.write("<version>1</version>"); 
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>"); 
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Linked Neighborhood Seed Predation</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<disperse>");
    oOut.write("<di_minDbhForReproduction>");
    oOut.write("<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>");
    oOut.write("<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>");
    oOut.write("</di_minDbhForReproduction>");
    oOut.write("<di_nonSpatialSlopeOfLambda>");
    oOut.write("<di_nssolVal species=\"Species_2\">0</di_nssolVal>");
    oOut.write("<di_nssolVal species=\"Species_1\">0</di_nssolVal>");
    oOut.write("</di_nonSpatialSlopeOfLambda>");
    oOut.write("<di_nonSpatialInterceptOfLambda>");
    oOut.write("<di_nsiolVal species=\"Species_1\">1</di_nsiolVal>");
    oOut.write("<di_nsiolVal species=\"Species_2\">1</di_nsiolVal>");
    oOut.write("</di_nonSpatialInterceptOfLambda>");
    oOut.write("<di_seedDistributionMethod>0</di_seedDistributionMethod>");
    oOut.write("</disperse>");
    oOut.write("<seedPredation>");
   
    oOut.write("<pr_funcRespPredatorInitialDensity>0.46875</pr_funcRespPredatorInitialDensity>");
    oOut.write("<pr_funcRespMaxInstDeclineRate1>-0.050622442</pr_funcRespMaxInstDeclineRate1>");
    oOut.write("<pr_funcRespMaxInstDeclineRate2>-1.050622442</pr_funcRespMaxInstDeclineRate2>");
    oOut.write("<pr_funcRespDemographicEfficiency1>0.154258</pr_funcRespDemographicEfficiency1>");
    oOut.write("<pr_funcRespDemographicEfficiency2>2.154258</pr_funcRespDemographicEfficiency2>");
    oOut.write("<pr_funcRespDensityDependentCoefficient1>-0.718667</pr_funcRespDensityDependentCoefficient1>");
    oOut.write("<pr_funcRespDensityDependentCoefficient2>-0.818667</pr_funcRespDensityDependentCoefficient2>");
    oOut.write("<pr_funcRespMaxIntakeRate>");
    oOut.write("<pr_frmirVal species=\"Species_1\">1042.2</pr_frmirVal>");
    oOut.write("<pr_frmirVal species=\"Species_2\">800.2</pr_frmirVal>");
    oOut.write("</pr_funcRespMaxIntakeRate>");
    oOut.write("<pr_funcRespForagingEfficiency>");
    oOut.write("<pr_frfeVal species=\"Species_1\">0.001389</pr_frfeVal>");
    oOut.write("<pr_frfeVal species=\"Species_2\">0.3389</pr_frfeVal>");
    oOut.write("</pr_funcRespForagingEfficiency>");
    oOut.write("<pr_funcRespNumWeeksSeedFall>12</pr_funcRespNumWeeksSeedFall>"); 
    oOut.write("<pr_funcRespWeekSeason2Starts>20</pr_funcRespWeekSeason2Starts>");
    oOut.write("<pr_funcRespNumWeeksToModel>52</pr_funcRespNumWeeksToModel>");
    oOut.write("<pr_funcRespWeekGerminationStarts>32</pr_funcRespWeekGerminationStarts>");
    oOut.write("<pr_funcRespPreservePredatorDensities>0</pr_funcRespPreservePredatorDensities>");
    oOut.write("<pr_funcRespProportionGerminating>0.35</pr_funcRespProportionGerminating>");
    oOut.write("<pr_funcRespOutputFilename>TestSeeds.txt</pr_funcRespOutputFilename>");
    
    oOut.write("<pr_funcRespPredatorInitialDensityLnk>0.002</pr_funcRespPredatorInitialDensityLnk>");
    oOut.write("<pr_funcRespMaxInstDeclineRate1Lnk>-0.04</pr_funcRespMaxInstDeclineRate1Lnk>");
    oOut.write("<pr_funcRespMaxInstDeclineRate2Lnk>-1.04</pr_funcRespMaxInstDeclineRate2Lnk>");
    oOut.write("<pr_funcRespDemographicEfficiency1Lnk>0.0002</pr_funcRespDemographicEfficiency1Lnk>");
    oOut.write("<pr_funcRespDemographicEfficiency2Lnk>0.3</pr_funcRespDemographicEfficiency2Lnk>");
    oOut.write("<pr_funcRespDensityDependentCoefficient1Lnk>-0.008</pr_funcRespDensityDependentCoefficient1Lnk>");
    oOut.write("<pr_funcRespDensityDependentCoefficient2Lnk>-0.8</pr_funcRespDensityDependentCoefficient2Lnk>");
    oOut.write("<pr_funcRespMaxIntakeRateLnk>");
    oOut.write("<pr_frmirlVal species=\"Species_1\">300</pr_frmirlVal>");
    oOut.write("<pr_frmirlVal species=\"Species_2\">245</pr_frmirlVal>");
    oOut.write("</pr_funcRespMaxIntakeRateLnk>");
    oOut.write("<pr_funcRespForagingEfficiencyLnk>");
    oOut.write("<pr_frfelVal species=\"Species_1\">0.002</pr_frfelVal>");
    oOut.write("<pr_frfelVal species=\"Species_2\">0.444</pr_frfelVal>");
    oOut.write("</pr_funcRespForagingEfficiencyLnk>");
    oOut.write("<pr_funcRespNumWeeksSeedFallLnk>7</pr_funcRespNumWeeksSeedFallLnk>");
    oOut.write("<pr_funcRespWeekSeason2StartsLnk>21</pr_funcRespWeekSeason2StartsLnk>");
    oOut.write("<pr_funcRespNumWeeksToModelLnk>30</pr_funcRespNumWeeksToModelLnk>");
    oOut.write("<pr_funcRespWeekGerminationStartsLnk>10</pr_funcRespWeekGerminationStartsLnk>");
    oOut.write("<pr_funcRespPreservePredatorDensitiesLnk>1</pr_funcRespPreservePredatorDensitiesLnk>");
    oOut.write("<pr_funcRespProportionGerminatingLnk>0.1</pr_funcRespProportionGerminatingLnk>");
    oOut.write("<pr_funcRespOutputFilenameLnk>TestLinkSeeds.txt</pr_funcRespOutputFilenameLnk>");
    
    oOut.write("<pr_neighPredMastingDensity>4</pr_neighPredMastingDensity>");
    oOut.write("<pr_neighPredMinNeighDBH>");
    oOut.write("<pr_npmndVal species=\"Species_1\">10</pr_npmndVal>");
    oOut.write("<pr_npmndVal species=\"Species_2\">15</pr_npmndVal>");
    oOut.write("</pr_neighPredMinNeighDBH>");
    oOut.write("<pr_neighPredCounts4Mast>");
    oOut.write("<pr_npc4mVal species=\"Species_1\">1</pr_npc4mVal>");
    oOut.write("<pr_npc4mVal species=\"Species_2\">0</pr_npc4mVal>");
    oOut.write("</pr_neighPredCounts4Mast>");
    oOut.write("<pr_neighPredRadius>15</pr_neighPredRadius>");
    oOut.write("<pr_neighPredMastingP0>");
    oOut.write("<pr_npmp0Val species=\"Species_1\">1.32</pr_npmp0Val>");
    oOut.write("<pr_npmp0Val species=\"Species_2\">0.86</pr_npmp0Val>");
    oOut.write("</pr_neighPredMastingP0>");
    oOut.write("<pr_neighPredMastingPnSpecies_1>");
    oOut.write("<pr_npmpnVal species=\"Species_1\">-5.94</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Species_2\">0</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnSpecies_1>");
    oOut.write("<pr_neighPredMastingPnSpecies_2>");
    oOut.write("<pr_npmpnVal species=\"Species_1\">2.8</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Species_2\">3.1</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnSpecies_2>");
    oOut.write("<pr_neighPredNonMastingP0>");
    oOut.write("<pr_npnmp0Val species=\"Species_1\">-2.51</pr_npnmp0Val>");
    oOut.write("<pr_npnmp0Val species=\"Species_2\">-1.36</pr_npnmp0Val>");
    oOut.write("</pr_neighPredNonMastingP0>");
    oOut.write("<pr_neighPredNonMastingPnSpecies_1>");
    oOut.write("<pr_npnmpnVal species=\"Species_1\">2.56</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Species_2\">0.23</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnSpecies_1>");
    oOut.write("<pr_neighPredNonMastingPnSpecies_2>");
    oOut.write("<pr_npnmpnVal species=\"Species_1\">4.91</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Species_2\">1.9</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnSpecies_2>");
    
    oOut.write("<pr_neighPredMinNeighDBHLnk>12</pr_neighPredMinNeighDBHLnk>");
    oOut.write("<pr_neighPredRadiusLnk>11</pr_neighPredRadiusLnk>");
    oOut.write("<pr_neighPredLnkP0>");
    oOut.write("<pr_npmlp0Val species=\"Species_1\">1.67</pr_npmlp0Val>");
    oOut.write("<pr_npmlp0Val species=\"Species_2\">1.83</pr_npmlp0Val>");
    oOut.write("</pr_neighPredLnkP0>");
    oOut.write("<pr_neighPredPnLnkSpecies_1>");
    oOut.write("<pr_nppnlVal species=\"Species_1\">-5.32</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Species_2\">-3.2</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkSpecies_1>");
    oOut.write("<pr_neighPredPnLnkSpecies_2>");
    oOut.write("<pr_nppnlVal species=\"Species_1\">2.1</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Species_2\">3.4</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkSpecies_2>");
    oOut.write("<pr_neighPredLnkMinNeighDBH>");
    oOut.write("<pr_nplmndVal species=\"Species_1\">13</pr_nplmndVal>");
    oOut.write("<pr_nplmndVal species=\"Species_2\">14</pr_nplmndVal>");
    oOut.write("</pr_neighPredLnkMinNeighDBH>");
    
    oOut.write("</seedPredation>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }
    
  /**
   * Writes a file with no seed predation settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>August 18, 2006: Created (LEM)
   */
  protected String WriteNoSeedPredationXMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>50</timesteps>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Constant GLI</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
  
  protected String WriteXMLFileForSpeciesCopy() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Western_Hemlock\" />");
    oOut.write("<tr_species speciesName=\"Western_Redcedar\" />");
    oOut.write("<tr_species speciesName=\"Amabilis_Fir\" />");
    oOut.write("<tr_species speciesName=\"Subalpine_Fir\" />");
    oOut.write("<tr_species speciesName=\"Hybrid_Spruce\" />");
    oOut.write("<tr_species speciesName=\"Lodgepole_Pine\" />");
    oOut.write("<tr_species speciesName=\"Trembling_Aspen\" />");
    oOut.write("<tr_species speciesName=\"Black_Cottonwood\" />");
    oOut.write("<tr_species speciesName=\"Paper_Birch\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Neighborhood Seed Predation</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Western_Hemlock\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Western_Redcedar\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Amabilis_Fir\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Subalpine_Fir\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<seedPredation>");
    oOut.write("<pr_neighPredMastingDensity>4</pr_neighPredMastingDensity>");
    oOut.write("<pr_neighPredMinNeighDBH>10</pr_neighPredMinNeighDBH>");
    oOut.write("<pr_neighPredRadius>15</pr_neighPredRadius>");
    oOut.write("<pr_neighPredMastingP0>");
    oOut.write("<pr_npmp0Val species=\"Western_Hemlock\">0.01</pr_npmp0Val>");
    oOut.write("<pr_npmp0Val species=\"Western_Redcedar\">0.02</pr_npmp0Val>");
    oOut.write("<pr_npmp0Val species=\"Amabilis_Fir\">0.03</pr_npmp0Val>");
    oOut.write("<pr_npmp0Val species=\"Subalpine_Fir\">0.04</pr_npmp0Val>");
    oOut.write("</pr_neighPredMastingP0>");
    oOut.write("<pr_neighPredMastingPnWestern_Hemlock>");
    oOut.write("<pr_npmpnVal species=\"Western_Hemlock\">0.05</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Western_Redcedar\">0.06</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Amabilis_Fir\">0.07</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Subalpine_Fir\">0.08</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnWestern_Hemlock>");
    oOut.write("<pr_neighPredMastingPnWestern_Redcedar>");
    oOut.write("<pr_npmpnVal species=\"Western_Hemlock\">0.09</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Western_Redcedar\">0.1</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Amabilis_Fir\">0.11</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Subalpine_Fir\">0.12</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnWestern_Redcedar>");
    oOut.write("<pr_neighPredMastingPnAmabilis_Fir>");
    oOut.write("<pr_npmpnVal species=\"Western_Hemlock\">0.13</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Western_Redcedar\">0.14</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Amabilis_Fir\">0.15</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Subalpine_Fir\">0.16</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnAmabilis_Fir>");
    oOut.write("<pr_neighPredMastingPnSubalpine_Fir>");
    oOut.write("<pr_npmpnVal species=\"Western_Hemlock\">0.17</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Western_Redcedar\">0.18</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Amabilis_Fir\">0.19</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Subalpine_Fir\">0.2</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnSubalpine_Fir>");
    oOut.write("<pr_neighPredMastingPnHybrid_Spruce>");
    oOut.write("<pr_npmpnVal species=\"Western_Hemlock\">0.21</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Western_Redcedar\">0.22</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Amabilis_Fir\">0.23</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Subalpine_Fir\">0.24</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnHybrid_Spruce>");
    oOut.write("<pr_neighPredMastingPnLodgepole_Pine>");
    oOut.write("<pr_npmpnVal species=\"Western_Hemlock\">0.25</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Western_Redcedar\">0.26</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Amabilis_Fir\">0.27</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Subalpine_Fir\">0.28</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnLodgepole_Pine>");
    oOut.write("<pr_neighPredMastingPnTrembling_Aspen>");
    oOut.write("<pr_npmpnVal species=\"Western_Hemlock\">0.29</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Western_Redcedar\">0.3</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Amabilis_Fir\">0.31</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Subalpine_Fir\">0.32</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnTrembling_Aspen>");
    oOut.write("<pr_neighPredMastingPnBlack_Cottonwood>");
    oOut.write("<pr_npmpnVal species=\"Western_Hemlock\">0.33</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Western_Redcedar\">0.34</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Amabilis_Fir\">0.35</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Subalpine_Fir\">0.36</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnBlack_Cottonwood>");
    oOut.write("<pr_neighPredMastingPnPaper_Birch>");
    oOut.write("<pr_npmpnVal species=\"Western_Hemlock\">0.37</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Western_Redcedar\">0.38</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Amabilis_Fir\">0.39</pr_npmpnVal>");
    oOut.write("<pr_npmpnVal species=\"Subalpine_Fir\">0.4</pr_npmpnVal>");
    oOut.write("</pr_neighPredMastingPnPaper_Birch>");
    oOut.write("<pr_neighPredNonMastingP0>");
    oOut.write("<pr_npnmp0Val species=\"Western_Hemlock\">1.01</pr_npnmp0Val>");
    oOut.write("<pr_npnmp0Val species=\"Western_Redcedar\">1.02</pr_npnmp0Val>");
    oOut.write("<pr_npnmp0Val species=\"Amabilis_Fir\">1.03</pr_npnmp0Val>");
    oOut.write("<pr_npnmp0Val species=\"Subalpine_Fir\">1.04</pr_npnmp0Val>");
    oOut.write("</pr_neighPredNonMastingP0>");
    oOut.write("<pr_neighPredNonMastingPnWestern_Hemlock>");
    oOut.write("<pr_npnmpnVal species=\"Western_Hemlock\">1.05</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Western_Redcedar\">1.06</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Amabilis_Fir\">1.07</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Subalpine_Fir\">1.08</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnWestern_Hemlock>");
    oOut.write("<pr_neighPredNonMastingPnWestern_Redcedar>");
    oOut.write("<pr_npnmpnVal species=\"Western_Hemlock\">1.09</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Western_Redcedar\">1.1</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Amabilis_Fir\">1.11</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Subalpine_Fir\">1.12</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnWestern_Redcedar>");
    oOut.write("<pr_neighPredNonMastingPnAmabilis_Fir>");
    oOut.write("<pr_npnmpnVal species=\"Western_Hemlock\">1.13</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Western_Redcedar\">1.14</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Amabilis_Fir\">1.15</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Subalpine_Fir\">1.16</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnAmabilis_Fir>");
    oOut.write("<pr_neighPredNonMastingPnSubalpine_Fir>");
    oOut.write("<pr_npnmpnVal species=\"Western_Hemlock\">1.17</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Western_Redcedar\">1.18</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Amabilis_Fir\">1.19</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Subalpine_Fir\">1.2</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnSubalpine_Fir>");
    oOut.write("<pr_neighPredNonMastingPnHybrid_Spruce>");
    oOut.write("<pr_npnmpnVal species=\"Western_Hemlock\">1.21</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Western_Redcedar\">1.22</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Amabilis_Fir\">1.23</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Subalpine_Fir\">1.24</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnHybrid_Spruce>");
    oOut.write("<pr_neighPredNonMastingPnLodgepole_Pine>");
    oOut.write("<pr_npnmpnVal species=\"Western_Hemlock\">1.25</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Western_Redcedar\">1.26</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Amabilis_Fir\">1.27</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Subalpine_Fir\">1.28</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnLodgepole_Pine>");
    oOut.write("<pr_neighPredNonMastingPnTrembling_Aspen>");
    oOut.write("<pr_npnmpnVal species=\"Western_Hemlock\">1.29</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Western_Redcedar\">1.3</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Amabilis_Fir\">1.31</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Subalpine_Fir\">1.32</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnTrembling_Aspen>");
    oOut.write("<pr_neighPredNonMastingPnBlack_Cottonwood>");
    oOut.write("<pr_npnmpnVal species=\"Western_Hemlock\">1.33</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Western_Redcedar\">1.34</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Amabilis_Fir\">1.35</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Subalpine_Fir\">1.36</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnBlack_Cottonwood>");
    oOut.write("<pr_neighPredNonMastingPnPaper_Birch>");
    oOut.write("<pr_npnmpnVal species=\"Western_Hemlock\">1.37</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Western_Redcedar\">1.38</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Amabilis_Fir\">1.39</pr_npnmpnVal>");
    oOut.write("<pr_npnmpnVal species=\"Subalpine_Fir\">1.4</pr_npnmpnVal>");
    oOut.write("</pr_neighPredNonMastingPnPaper_Birch>");
    
    oOut.write("<pr_neighPredPnLnkWestern_Hemlock>");
    oOut.write("<pr_nppnlVal species=\"Western_Hemlock\">1.41</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Western_Redcedar\">1.42</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Amabilis_Fir\">1.43</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Subalpine_Fir\">1.44</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkWestern_Hemlock>");
    oOut.write("<pr_neighPredPnLnkWestern_Redcedar>");
    oOut.write("<pr_nppnlVal species=\"Western_Hemlock\">1.45</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Western_Redcedar\">1.46</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Amabilis_Fir\">1.47</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Subalpine_Fir\">1.48</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkWestern_Redcedar>");
    oOut.write("<pr_neighPredPnLnkAmabilis_Fir>");
    oOut.write("<pr_nppnlVal species=\"Western_Hemlock\">1.49</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Western_Redcedar\">1.5</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Amabilis_Fir\">1.51</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Subalpine_Fir\">1.52</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkAmabilis_Fir>");
    oOut.write("<pr_neighPredPnLnkSubalpine_Fir>");
    oOut.write("<pr_nppnlVal species=\"Western_Hemlock\">1.53</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Western_Redcedar\">1.54</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Amabilis_Fir\">1.55</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Subalpine_Fir\">1.56</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkSubalpine_Fir>");
    oOut.write("<pr_neighPredPnLnkHybrid_Spruce>");
    oOut.write("<pr_nppnlVal species=\"Western_Hemlock\">1.57</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Western_Redcedar\">1.58</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Amabilis_Fir\">1.59</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Subalpine_Fir\">1.60</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkHybrid_Spruce>");
    oOut.write("<pr_neighPredPnLnkLodgepole_Pine>");
    oOut.write("<pr_nppnlVal species=\"Western_Hemlock\">1.61</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Western_Redcedar\">1.62</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Amabilis_Fir\">1.63</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Subalpine_Fir\">1.64</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkLodgepole_Pine>");
    oOut.write("<pr_neighPredPnLnkTrembling_Aspen>");
    oOut.write("<pr_nppnlVal species=\"Western_Hemlock\">1.65</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Western_Redcedar\">1.66</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Amabilis_Fir\">1.67</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Subalpine_Fir\">1.68</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkTrembling_Aspen>");
    oOut.write("<pr_neighPredPnLnkBlack_Cottonwood>");
    oOut.write("<pr_nppnlVal species=\"Western_Hemlock\">1.69</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Western_Redcedar\">1.7</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Amabilis_Fir\">1.71</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Subalpine_Fir\">1.72</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkBlack_Cottonwood>");
    oOut.write("<pr_neighPredPnLnkPaper_Birch>");
    oOut.write("<pr_nppnlVal species=\"Western_Hemlock\">1.73</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Western_Redcedar\">1.74</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Amabilis_Fir\">1.75</pr_nppnlVal>");
    oOut.write("<pr_nppnlVal species=\"Subalpine_Fir\">1.76</pr_nppnlVal>");
    oOut.write("</pr_neighPredPnLnkPaper_Birch>");
    
    oOut.write("</seedPredation>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}